@@ -1,53 +1,37 @@
 /* -*-c-*- */
-/* This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+/* This module is based on Twm, but has been siginificantly modified
+ * by Rob Nation */
+/*
+ *       Copyright 1988 by Evans & Sutherland Computer Corporation,
+ *                          Salt Lake City, Utah
+ *  Portions Copyright 1989 by the Massachusetts Institute of Technology
+ *                        Cambridge, Massachusetts
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ *                           All Rights Reserved
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *    Permission to use, copy, modify, and distribute this software and
+ *    its documentation  for  any  purpose  and  without  fee is hereby
+ *    granted, provided that the above copyright notice appear  in  all
+ *    copies and that both  that  copyright  notice  and  this  permis-
+ *    sion  notice appear in supporting  documentation,  and  that  the
+ *    names of Evans & Sutherland and M.I.T. not be used in advertising
+ *    in publicity pertaining to distribution of the  software  without
+ *    specific, written prior permission.
+ *
+ *    EVANS & SUTHERLAND AND M.I.T. DISCLAIM ALL WARRANTIES WITH REGARD
+ *    TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES  OF  MERCHANT-
+ *    ABILITY  AND  FITNESS,  IN  NO  EVENT SHALL EVANS & SUTHERLAND OR
+ *    M.I.T. BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL  DAM-
+ *    AGES OR  ANY DAMAGES WHATSOEVER  RESULTING FROM LOSS OF USE, DATA
+ *    OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+ *    TORTIOUS ACTION, ARISING OUT OF OR IN  CONNECTION  WITH  THE  USE
+ *    OR PERFORMANCE OF THIS SOFTWARE.
  */
-/****************************************************************************
- * This module is based on Twm, but has been siginificantly modified
- * by Rob Nation
- ****************************************************************************/
-/*****************************************************************************/
-/**       Copyright 1988 by Evans & Sutherland Computer Corporation,        **/
-/**                          Salt Lake City, Utah                           **/
-/**  Portions Copyright 1989 by the Massachusetts Institute of Technology   **/
-/**                        Cambridge, Massachusetts                         **/
-/**                                                                         **/
-/**                           All Rights Reserved                           **/
-/**                                                                         **/
-/**    Permission to use, copy, modify, and distribute this software and    **/
-/**    its documentation  for  any  purpose  and  without  fee is hereby    **/
-/**    granted, provided that the above copyright notice appear  in  all    **/
-/**    copies and that both  that  copyright  notice  and  this  permis-    **/
-/**    sion  notice appear in supporting  documentation,  and  that  the    **/
-/**    names of Evans & Sutherland and M.I.T. not be used in advertising    **/
-/**    in publicity pertaining to distribution of the  software  without    **/
-/**    specific, written prior permission.                                  **/
-/**                                                                         **/
-/**    EVANS & SUTHERLAND AND M.I.T. DISCLAIM ALL WARRANTIES WITH REGARD    **/
-/**    TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES  OF  MERCHANT-    **/
-/**    ABILITY  AND  FITNESS,  IN  NO  EVENT SHALL EVANS & SUTHERLAND OR    **/
-/**    M.I.T. BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL  DAM-    **/
-/**    AGES OR  ANY DAMAGES WHATSOEVER  RESULTING FROM LOSS OF USE, DATA    **/
-/**    OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER    **/
-/**    TORTIOUS ACTION, ARISING OUT OF OR IN  CONNECTION  WITH  THE  USE    **/
-/**    OR PERFORMANCE OF THIS SOFTWARE.                                     **/
-/*****************************************************************************/
 
 #ifndef FVWM_H
 #define FVWM_H
 
-/* ---------------------------- included header files ----------------------- */
+/* ---------------------------- included header files ---------------------- */
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -56,27 +40,15 @@
 #include "libs/PictureBase.h"
 #include "libs/Flocale.h"
 #include "window_flags.h"
+#include "condrc.h"
 
-/* ---------------------------- global definitions -------------------------- */
+/* ---------------------------- global definitions ------------------------- */
 
 #ifndef WithdrawnState
 #define WithdrawnState 0
 #endif
 
-#if RETSIGTYPE != void
-#define SIGNAL_RETURN return 0
-#else
-#define SIGNAL_RETURN return
-#endif
-
-#ifndef TRUE
-#define TRUE    1
-#define FALSE   0
-#endif
-
-#define NULLSTR ((char *) NULL)
-
-/* ---------------------------- global macros ------------------------------- */
+/* ---------------------------- global macros ------------------------------ */
 
 /*
  * Fvwm trivia: There were 97 commands in the fvwm command table
@@ -86,7 +58,7 @@
 
 /* Macro for args passed to fvwm commands... */
 #define F_CMD_ARGS \
-	fvwm_cond_func_rc *cond_rc, const exec_context_t *exc, char *action
+	cond_rc_t *cond_rc, const exec_context_t *exc, char *action
 #define F_PASS_ARGS cond_rc, exc, action
 #define FUNC_FLAGS_TYPE unsigned char
 
@@ -103,7 +75,7 @@
 #define FW_W_ICON_PIXMAP(fw)  ((fw)->wins.icon_pixmap_w)
 #define FW_W_TRANSIENTFOR(fw) ((fw)->wins.transientfor)
 
-/* ---------------------------- forward declarations ------------------------ */
+/* ---------------------------- forward declarations ----------------------- */
 
 struct exec_context_t;
 
@@ -112,7 +84,7 @@ struct exec_context_t;
 struct FvwmDecor;
 #endif
 
-/* ---------------------------- type definitions ---------------------------- */
+/* ---------------------------- type definitions --------------------------- */
 
 /* This structure carries information about the initial window state and
  * placement.  This information is gathered at various places: the (re)capture
@@ -128,8 +100,8 @@ struct FvwmDecor;
  *
  * do_override_ppos
  *   This flag is used in PlaceWindow().  If it is set, the position requested
- *   by the program is ignored unconditionally.  This is used during the initial
- *   capture and later recapture operations.
+ *   by the program is ignored unconditionally.  This is used during the
+ *   initial capture and later recapture operations.
  *
  * is_iconified_by_parent
  *   Preserves the information if the window is a transient window that was
@@ -173,15 +145,7 @@ typedef struct
 	int initial_icon_y;
 	int default_icon_x;
 	int default_icon_y;
-} initial_window_options_type;
-
-typedef enum
-{
-	COND_RC_BREAK = -2,
-	COND_RC_ERROR = -1,
-	COND_RC_NO_MATCH = 0,
-	COND_RC_OK = 1
-} fvwm_cond_func_rc;
+} initial_window_options_t;
 
 /*
   For 1 style statement, there can be any number of IconBoxes.
@@ -229,14 +193,15 @@ typedef struct
 typedef struct
 {
 	/* common flags (former flags in bits 0-12) */
-	unsigned is_sticky_on_page : 1;
-	unsigned is_sticky_on_desk : 1;
+	unsigned is_sticky_across_pages : 1;
+	unsigned is_sticky_across_desks : 1;
 	unsigned has_icon_font : 1;
 	unsigned has_no_border : 1;
 	unsigned has_window_font : 1;
 	unsigned title_dir : 2;
-	/* static flags that do not change dynamically after the window has been
-	 * created */
+	unsigned user_states : 32;
+	/* static flags that do not change dynamically after the window has
+	 * been created */
 	struct
 	{
 		unsigned do_circulate_skip : 1;
@@ -248,6 +213,7 @@ typedef struct
 		unsigned do_ewmh_ignore_strut_hints : 1;
 		unsigned do_ewmh_mini_icon_override : 1;
 		unsigned do_ewmh_use_stacking_hints : 1;
+		unsigned do_ewmh_ignore_window_type : 1;
 		unsigned do_iconify_window_groups : 1;
 		unsigned do_ignore_gnome_hints : 1;
 		unsigned do_ignore_icon_boxes : 1;
@@ -268,6 +234,10 @@ typedef struct
 		unsigned has_no_icon_title : 1;
 		unsigned has_override_size : 1;
 		unsigned has_stippled_title : 1;
+		unsigned has_stippled_icon_title : 1;
+		/* default has to be 0, therefore no_, not in macros */
+		unsigned has_no_sticky_stippled_title : 1;
+		unsigned has_no_sticky_stippled_icon_title : 1;
 		unsigned icon_override : 2;
 #define NO_ACTIVE_ICON_OVERRIDE 0
 #define ICON_OVERRIDE           1
@@ -276,8 +246,12 @@ typedef struct
 		unsigned is_bottom_title_rotated : 1;
 		unsigned is_fixed : 1;
 		unsigned is_fixed_ppos : 1;
-		unsigned is_icon_sticky_on_page : 1;
-		unsigned is_icon_sticky_on_desk : 1;
+		unsigned is_uniconifiable : 1;
+		unsigned is_unmaximizable : 1;
+		unsigned is_unclosable : 1;
+		unsigned is_maximize_fixed_size_disallowed : 1;
+		unsigned is_icon_sticky_across_pages : 1;
+		unsigned is_icon_sticky_across_desks : 1;
 		unsigned is_icon_suppressed : 1;
 		unsigned is_left_title_rotated_cw : 1; /* cw = clock wise */
 		unsigned is_lenient : 1;
@@ -293,18 +267,26 @@ typedef struct
 #define WINDOWSHADE_BUSY          2
 #define WINDOWSHADE_LAZY_MASK   0x3
 		unsigned windowshade_laziness : 2;
+		unsigned use_title_decor_rotation : 1;
 		focus_policy_t focus_policy;
 	} s;
-} common_flags_type;
+} common_flags_t;
 
 typedef struct
 {
-	common_flags_type common;
+	common_flags_t common;
+#define CR_MOTION_METHOD_AUTO        0
+#define CR_MOTION_METHOD_USE_GRAV    1
+#define CR_MOTION_METHOD_STATIC_GRAV 2
+#define CR_MOTION_METHOD_MASK      0x3
+	unsigned cr_motion_method : 2;
+	unsigned was_cr_motion_method_detected : 1;
 	unsigned does_wm_delete_window : 1;
 	unsigned does_wm_take_focus : 1;
 	unsigned do_force_next_cr : 1;
 	unsigned do_force_next_pn : 1;
 	unsigned do_iconify_after_map : 1;
+	unsigned do_disable_constrain_size_fullscreen : 1;
 	/* Reuse this struct, don't free it, when destroying/recapturing
 	 * window. */
 	unsigned do_reuse_destroyed : 1;
@@ -360,7 +342,6 @@ typedef struct
 	unsigned is_partially_visible : 1;
 	/* is the icon pixmap ours to free? */
 	unsigned is_pixmap_ours : 1;
-	unsigned is_placed_wb3 : 1;
 	/* fvwm places the window itself */
 	unsigned is_placed_by_fvwm : 1;
 	/* mark window to be destroyed after last complex func has finished. */
@@ -398,9 +379,11 @@ typedef struct
 	/* says if the app have an ewmh icon of acceptable size for a mini
 	 * icon in its list of icons */
 	unsigned has_ewmh_mini_icon : 1;
+	unsigned has_ewmh_wm_pid : 1;
 	/* the ewmh icon is used as icon pixmap */
 	unsigned use_ewmh_icon : 1;
 	unsigned is_ewmh_modal : 1;
+	unsigned is_ewmh_fullscreen : 1;
 #define EWMH_STATE_UNDEFINED_HINT 0
 #define EWMH_STATE_NO_HINT        1
 #define EWMH_STATE_HAS_HINT       2
@@ -414,25 +397,64 @@ typedef struct
 	unsigned has_ewmh_init_skip_taskbar_state : 2;
 	unsigned has_ewmh_init_sticky_state : 2;
 	unsigned has_ewmh_init_wm_desktop : 2;
-	unsigned user_states : 32;
 } window_flags;
 
+
+/* Actions allowed by modules. */
+typedef struct action_flags
+{
+	unsigned is_movable : 1;
+	unsigned is_deletable : 1;
+	unsigned is_destroyable : 1;
+	unsigned is_closable : 1;
+	unsigned is_maximizable : 1;
+	unsigned is_resizable : 1;
+	unsigned is_iconifiable : 1;
+} action_flags;
+
+
+/* Window name data structure for window conditions: a list of lists
+   of names to match, the boolean operation on the matches being an
+   AND of ORs. */
+struct namelist			/* matches to names in this list are ORed */
+{
+	char *name;
+	struct namelist *next;
+};
+
+struct name_condition		/* matches to namelists in this list are
+				   ANDed, after possibly inverting each */
+{
+	Bool invert;
+	struct namelist *namelist;
+	struct name_condition *next;
+};
+
 /* Window mask for Circulate and Direction functions */
 typedef struct WindowConditionMask
 {
 	struct
 	{
 		unsigned do_accept_focus : 1;
+		unsigned do_check_desk : 1;
+		unsigned do_check_desk_and_global_page : 1;
+		unsigned do_check_desk_and_page : 1;
+		unsigned do_check_global_page : 1;
+		unsigned do_check_overlapped : 1;
+		unsigned do_check_page : 1;
+		unsigned do_not_check_screen : 1;
 		unsigned needs_current_desk : 1;
-		unsigned needs_current_page : 1;
+		unsigned needs_current_desk_and_global_page : 1;
+		unsigned needs_current_desk_and_page : 1;
 		unsigned needs_current_global_page : 1;
-#define NEEDS_ANY        0
-#define NEEDS_TRUE       1
-#define NEEDS_FALSE      2
+		unsigned needs_current_page : 1;
+#define NEEDS_ANY   0
+#define NEEDS_TRUE  1
+#define NEEDS_FALSE 2
 		unsigned needs_focus : 2;
-		unsigned needs_name : 1;
-		unsigned needs_not_name : 1;
+		unsigned needs_overlapped : 2;
 		unsigned needs_pointer : 2;
+		unsigned needs_same_layer : 1;
 		unsigned use_circulate_hit : 1;
 		unsigned use_circulate_hit_icon : 1;
 		unsigned use_circulate_hit_shaded : 1;
@@ -440,14 +462,16 @@ typedef struct WindowConditionMask
 	} my_flags;
 	window_flags flags;
 	window_flags flag_mask;
-	char *name;
+	struct name_condition *name_condition;
 	int layer;
+	int placed_by_button_mask;
+	int placed_by_button_set_mask;
 } WindowConditionMask;
 
-/* only style.c and add_window.c are allowed to access this struct!! */
+/* only style.c and add_window.c are allowed to access this struct! */
 typedef struct
 {
-	common_flags_type common;
+	common_flags_t common;
 	unsigned do_decorate_transient : 1;
 	/* old placement flags */
 #define PLACE_DUMB            0x0
@@ -457,16 +481,16 @@ typedef struct
 #define PLACE_RANDOM          0x4
 	/* new placements value, try to get a minimal backward compatibility
 	 * with the old flags:
-	 * Dumb+Active=Manual,
-	 * Dumb+Random=Cascade,
-	 * Smart+Random= TileCascade,
-	 * Smart+Active=TileManual,
-	 * Random+Smart+Clever=MINOVERLAP which is the original Clever
+	 * Dumb+Active = Manual,
+	 * Dumb+Random = Cascade,
+	 * Smart+Random = TileCascade,
+	 * Smart+Active = TileManual,
+	 * Random+Smart+Clever = MINOVERLAP which is the original Clever
 	 * placement code,
-	 * Active+Smart+Clever= MINOVERLAPPERCENT which is the "new" Clever
+	 * Active+Smart+Clever = MINOVERLAPPERCENT which is the "new" Clever
 	 * placement code and was the original Clever placement code. Now the
 	 * original placement code said:
-	 * Active/Random+Dumb+Clever=Active/Random+Dumb (with Dumb Clever is
+	 * Active/Random+Dumb+Clever = Active/Random+Dumb (with Dumb Clever is
 	 * ignored); These represent the not use value: 0x2=Active+Dumb+Clever,
 	 * 0x6=Random+Dumb+Clever */
 #define PLACE_MANUAL            0x0
@@ -477,9 +501,16 @@ typedef struct
 #define PLACE_TILECASCADE       0x5
 #define PLACE_CASCADE_B         0x6
 #define PLACE_MINOVERLAP        0x7
-#define PLACE_MASK              0x7
-	unsigned placement_mode : 3;
+#define PLACE_CENTER            0x8
+#define PLACE_UNDERMOUSE        0x9
+#define PLACE_MASK              0xF
+	unsigned placement_mode : 4;
 	unsigned ewmh_placement_mode : 2; /* see ewmh.h */
+#define WS_CR_MOTION_METHOD_AUTO CR_MOTION_METHOD_AUTO
+#define WS_CR_MOTION_METHOD_USE_GRAV CR_MOTION_METHOD_USE_GRAV
+#define WS_CR_MOTION_METHOD_STATIC_GRAV CR_MOTION_METHOD_STATIC_GRAV
+#define WS_CR_MOTION_METHOD_MASK CR_MOTION_METHOD_MASK
+	unsigned ws_cr_motion_method : 2;
 	unsigned do_save_under : 1;
 	unsigned do_start_iconic : 1;
 	unsigned do_start_lowered : 1;
@@ -494,6 +525,9 @@ typedef struct
 	unsigned has_icon_boxes : 1;
 	unsigned has_icon_size_limits : 1;
 	unsigned has_max_window_size : 1;
+	unsigned has_icon_background_padding : 1;
+	unsigned has_icon_background_relief : 1;
+	unsigned has_icon_title_relief : 1;
 	unsigned has_window_shade_steps : 1;
 	unsigned has_mini_icon : 1;
 	unsigned has_mwm_decor : 1;
@@ -504,7 +538,8 @@ typedef struct
 #if 0
 	unsigned has_condition_mask : 1;
 #endif
-	unsigned is_button_disabled : NUMBER_OF_BUTTONS;
+	unsigned is_button_disabled : NUMBER_OF_TITLE_BUTTONS;
+	unsigned is_unmanaged : 1;
 #define BACKINGSTORE_DEFAULT 0
 #define BACKINGSTORE_ON      1
 #define BACKINGSTORE_OFF     2
@@ -515,6 +550,9 @@ typedef struct
 	unsigned use_colorset_hi : 1;
 	unsigned use_border_colorset : 1;
 	unsigned use_border_colorset_hi : 1;
+	unsigned use_icon_title_colorset : 1;
+	unsigned use_icon_title_colorset_hi : 1;
+	unsigned use_icon_background_colorset : 1;
 	unsigned use_layer : 1;
 	unsigned use_no_pposition : 1;
 	unsigned use_no_usposition : 1;
@@ -530,12 +568,24 @@ typedef struct
 	unsigned has_placement_percentage_penalty : 1;
 } style_flags;
 
-/* only style.c and add_window.c are allowed to access this struct!! */
+typedef struct
+{
+	char *name;
+	XID window_id;
+	struct
+	{
+		unsigned has_name:1;
+		unsigned has_window_id:1;
+	} flags;
+} style_id_t;
+
+
+/* only style.c and add_window.c are allowed to access this struct! */
 typedef struct window_style
 {
 	struct window_style *next;
 	struct window_style *prev;
-	char *name;
+	style_id_t id;
 #if 0
 	WindowConditionMask *condition_mask;
 #endif
@@ -548,6 +598,15 @@ typedef struct window_style
 	unsigned char max_icon_width;
 	unsigned char min_icon_height;
 	unsigned char max_icon_height;
+#define ICON_RESIZE_TYPE_NONE      0x0
+#define ICON_RESIZE_TYPE_STRETCHED 0x1
+#define ICON_RESIZE_TYPE_ADJUSTED  0x2
+#define ICON_RESIZE_TYPE_SHRUNK    0x3
+#define ICON_RESIZE_TYPE_MASK      0x3
+	unsigned icon_resize_type : 2;
+	unsigned char icon_background_padding;
+	char icon_background_relief;
+	char icon_title_relief;
 	char *icon_font;
 	char *window_font;
 	char *fore_color_name;
@@ -558,6 +617,9 @@ typedef struct window_style
 	int colorset_hi;
 	int border_colorset;
 	int border_colorset_hi;
+	int icon_title_colorset;
+	int icon_title_colorset_hi;
+	int icon_background_colorset;
 	short border_width;
 	/* resize handle width */
 	short handle_width;
@@ -580,6 +642,21 @@ typedef struct window_style
 	unsigned has_style_changed : 1;
 } window_style;
 
+typedef struct window_g
+{
+	rectangle frame;
+	/* absolute geometry when not maximized */
+	rectangle normal;
+	/* maximized window geometry */
+	rectangle max;
+	/* defect between maximized geometry before and after
+	 * constraining size. */
+	size_rect max_defect;
+	/* original delta between normalized and maximized window,
+	 * used to keep unmaximized window at same screen position */
+	position max_offset;
+} window_g;
+
 /* for each window that is on the display, one of these structures
  * is allocated and linked into a list
  */
@@ -591,6 +668,9 @@ typedef struct FvwmWindow
 	FlocaleNameString icon_name;
 	char *visible_name;
 	char *visible_icon_name;
+	/* if non-null: Use this instead of any other names for matching
+	   styles */
+	char *style_name;
 	int name_count;
 	int icon_name_count;
 	/* next fvwm window */
@@ -614,7 +694,7 @@ typedef struct FvwmWindow
 		Window client;
 		/* the title bar window and button windows */
 		Window title;
-		Window button_w[NUMBER_OF_BUTTONS];
+		Window button_w[NUMBER_OF_TITLE_BUTTONS];
 		/* sides of the border */
 		Window sides[4];
 		/* corner pieces */
@@ -628,17 +708,18 @@ typedef struct FvwmWindow
 	window_flags flags;
 	struct
 	{
-		unsigned buttons_drawn : NUMBER_OF_BUTTONS;
-		unsigned buttons_lit : NUMBER_OF_BUTTONS;
-		unsigned buttons_inverted : NUMBER_OF_BUTTONS;
-		unsigned buttons_toggled : NUMBER_OF_BUTTONS;
+		unsigned buttons_drawn : NUMBER_OF_TITLE_BUTTONS;
+		unsigned buttons_lit : NUMBER_OF_TITLE_BUTTONS;
+		unsigned buttons_inverted : NUMBER_OF_TITLE_BUTTONS;
+		unsigned buttons_toggled : NUMBER_OF_TITLE_BUTTONS;
 		unsigned parts_drawn : 12;
 		unsigned parts_lit : 12;
 		unsigned parts_inverted : 12;
 	} decor_state;
 	int nr_left_buttons;
 	int nr_right_buttons;
-#define BUTTON_INDEX(b) (((b) == 0) ? (NUMBER_OF_BUTTONS - 1) : ((b) - 1))
+#define BUTTON_INDEX(b) \
+	(((b) == 0) ? (NUMBER_OF_TITLE_BUTTONS - 1) : ((b) - 1))
 #ifdef USEDECOR
 	struct FvwmDecor *decor;
 #endif
@@ -649,6 +730,7 @@ typedef struct FvwmWindow
 	/* Note: if the type of this variable is changed, do update the
 	 * CONFIGARGSNEW macro in module_interface.c, libs/vpacket.h too! */
 	short boundary_width;
+	short unshaped_boundary_width;
 	short corner_width;
 	short visual_corner_width;
 
@@ -658,9 +740,10 @@ typedef struct FvwmWindow
 	short title_text_offset;
 	short title_length;
 	/* Note: if the type of this variable is changed, do update the
-	 * CONFIGARGSNEW macro in module_interface.c, libs/vpacket.h and too! */
+	 * CONFIGARGSNEW macro in module_interface.c, libs/vpacket.h and too!
+	 */
 	short title_thickness;
-	rotation_type title_text_rotation;
+	rotation_t title_text_rotation;
 	struct
 	{
 		/* geometry of the icon picture window */
@@ -670,6 +753,7 @@ typedef struct FvwmWindow
 		/* width of the text in the icon title */
 		int title_text_width;
 	} icon_g;
+	short icon_border_width;
 
 	/* Drawable depth for the icon */
 	int iconDepth;
@@ -696,14 +780,20 @@ typedef struct FvwmWindow
 	} attr_backup;
 	/* normal hints */
 	XSizeHints hints;
+	struct
+	{
+		int width_inc;
+		int height_inc;
+	} orig_hints;
 	/* WM hints */
 	XWMHints *wmhints;
 	XClassHint class;
 	/* Tells which desktop this window is on */
 	/* Note: if the type of this variable is changed, do update the
-	 * CONFIGARGSNEW macro in module_interface.c, libs/vpacket.h and too! */
+	 * CONFIGARGSNEW macro in module_interface.c, libs/vpacket.h and too!
+	 */
 	int Desk;
-	/* Where (if at all) was it focussed */
+	/* Where (if at all) was it focused */
 	int FocusDesk;
 	/* Desk to deiconify to, for StubbornIcons */
 	int DeIconifyDesk;
@@ -712,18 +802,8 @@ typedef struct FvwmWindow
 	FvwmPicture *mini_icon;
 	char *icon_bitmap_file;
 
-	rectangle frame_g;
-	/* absolute geometry when not maximized */
-	rectangle normal_g;
-	/* maximized window geometry */
-	rectangle max_g;
-	/* defect between maximized geometry before and after constraining
-	 * size. */
-	size_rect max_g_defect;
-	/* original delta between normalized and maximized window, used to
-	 * keep unmaximized window at same screen position */
-	position max_offset;
-	int *mwm_hints;
+	struct window_g g;
+	long *mwm_hints;
 	int ol_hints;
 	int functions;
 	/* Colormap windows property */
@@ -739,6 +819,9 @@ typedef struct FvwmWindow
 	int cs_hi;
 	int border_cs;
 	int border_cs_hi;
+	int icon_title_cs;
+	int icon_title_cs_hi;
+	int icon_background_cs;
 
 	unsigned long buttons;
 	/* zero or more iconboxes */
@@ -746,12 +829,20 @@ typedef struct FvwmWindow
 
 	int default_layer;
 	/* Note: if the type of this variable is changed, do update the
-	 * CONFIGARGSNEW macro in module_interface.c, libs/vpacket.h and too! */
+	 * CONFIGARGSNEW macro in module_interface.c, libs/vpacket.h and too!
+	 */
 	int layer;
+
 	unsigned char min_icon_width;
 	unsigned char max_icon_width;
 	unsigned char min_icon_height;
 	unsigned char max_icon_height;
+	unsigned short icon_resize_type;
+
+	unsigned char icon_background_padding;
+	char icon_background_relief;
+	char icon_title_relief;
+
 	int max_window_width;
 	int max_window_height;
 	int shade_anim_steps;
@@ -766,6 +857,8 @@ typedef struct FvwmWindow
 	float placement_penalty[6];
 	int placement_percentage_penalty[4];
 
+	unsigned char placed_by_button;
+
 #define EWMH_WINDOW_TYPE_NONE_ID      0
 #define EWMH_WINDOW_TYPE_DESKTOP_ID   1
 #define EWMH_WINDOW_TYPE_DIALOG_ID    2
@@ -774,7 +867,8 @@ typedef struct FvwmWindow
 #define EWMH_WINDOW_TYPE_NORMAL_ID    5
 #define EWMH_WINDOW_TYPE_TOOLBAR_ID   6
 	/* Note: if the type of this variable is changed, do update the
-	 * CONFIGARGSNEW macro in module_interface.c, libs/vpacket.h and too! */
+	 * CONFIGARGSNEW macro in module_interface.c, libs/vpacket.h and too!
+	 */
 	int ewmh_window_type;
 	/* icon geometry */
 	rectangle ewmh_icon_geometry;
@@ -790,8 +884,10 @@ typedef struct FvwmWindow
 	int ewmh_mini_icon_width;
 	/* memory for the initial _NET_WM_STATE */
 	/* Note: if the type of this variable is changed, do update the
-	 * CONFIGARGSNEW macro in module_interface.c, libs/vpacket.h and too! */
+	 * CONFIGARGSNEW macro in module_interface.c, libs/vpacket.h and too!
+	 */
 	int ewmh_hint_layer;
+	int ewmh_normal_layer; /* for restoring non ewmh layer */
 	/* memory for the initial _NET_WM_STATE */
 	unsigned long ewmh_hint_desktop;
 
@@ -803,9 +899,9 @@ typedef struct FvwmWindow
 	} scratch;
 } FvwmWindow;
 
-/* ---------------------------- exported variables (globals) ---------------- */
+/* ---------------------------- exported variables (globals) --------------- */
 
-/* ---------------------------- interface functions ------------------------- */
+/* ---------------------------- interface functions ------------------------ */
 
 void SetMWM_INFO(Window window);
 
