@@ -78,12 +78,12 @@
  */
 static int edge_thickness = 2;
 static int last_edge_thickness = 2;
-static unsigned int prev_page_x = 0;
-static unsigned int prev_page_y = 0;
+static int prev_page_x = 0;
+static int prev_page_y = 0;
 static int prev_desk = 0;
 static int prev_desk_and_page_desk = 0;
-static unsigned int prev_desk_and_page_page_x = 0;
-static unsigned int prev_desk_and_page_page_y = 0;
+static int prev_desk_and_page_page_x = 0;
+static int prev_desk_and_page_page_y = 0;
 
 /* ---------------------------- exported variables (globals) --------------- */
 
@@ -107,10 +107,11 @@ static unsigned int prev_desk_and_page_page_y = 0;
  * read (or if action is empty).
  *
  */
-static int GetDeskNumber(char *action)
+static int GetDeskNumber(char *action, int current_desk)
 {
 	int n;
 	int m;
+	int is_relative;
 	int desk;
 	int val[4];
 	int min, max;
@@ -119,7 +120,6 @@ static int GetDeskNumber(char *action)
 	{
 		return prev_desk;
 	}
-
 	n = GetIntegerArguments(action, NULL, &(val[0]), 4);
 	if (n <= 0)
 	{
@@ -127,23 +127,22 @@ static int GetDeskNumber(char *action)
 	}
 	if (n == 1)
 	{
-		return Scr.CurrentDesk + val[0];
+		return current_desk + val[0];
 	}
-
-	desk = Scr.CurrentDesk;
+	desk = current_desk;
 	m = 0;
-
 	if (val[0] == 0)
 	{
 		/* absolute desk number */
 		desk = val[1];
+		is_relative = 0;
 	}
 	else
 	{
 		/* relative desk number */
 		desk += val[0];
+		is_relative = 1;
 	}
-
 	if (n == 3)
 	{
 		m = 1;
@@ -152,8 +151,6 @@ static int GetDeskNumber(char *action)
 	{
 		m = 2;
 	}
-
-
 	if (n > 2)
 	{
 		/* handle limits */
@@ -164,13 +161,25 @@ static int GetDeskNumber(char *action)
 		}
 		else
 		{
-			/*  min > max is nonsense, so swap 'em.  */
+			/* min > max is nonsense, so swap 'em. */
 			min = val[m+1];
 			max = val[m];
 		}
-		if (desk < min)
+		if (is_relative)
 		{
-			/*  Relative move outside of range, wrap around.  */
+			/* Relative moves wrap around.	*/
+			if (desk < min)
+			{
+				desk += (max - min + 1);
+			}
+			else if (desk > max)
+			{
+				desk -= (max - min + 1);
+			}
+		}
+		else if (desk < min)
+		{
+			/* Relative move outside of range, wrap around. */
 			if (val[0] < 0)
 			{
 				desk = max;
@@ -182,7 +191,7 @@ static int GetDeskNumber(char *action)
 		}
 		else if (desk > max)
 		{
-			/*  Relative move outside of range, wrap around.  */
+			/* Move outside of range, truncate. */
 			if (val[0] > 0)
 			{
 				desk = min;
@@ -246,6 +255,7 @@ static void unmap_window(FvwmWindow *t)
 	if (ret)
 	{
 		XSelectInput(dpy, FW_W(t), eventMask);
+		XFlush(dpy);
 	}
 
 	return;
@@ -308,6 +318,7 @@ static void map_window(FvwmWindow *t)
 	if (ret)
 	{
 		XSelectInput(dpy, FW_W(t), eventMask);
+		XFlush(dpy);
 	}
 
 	return;
@@ -470,7 +481,7 @@ int HandlePaging(
 	int *delta_x, int *delta_y, Bool Grab, Bool fLoop,
 	Bool do_continue_previous)
 {
-	static unsigned int add_time = 0;
+	static int add_time = 0;
 	int x,y;
 	XEvent e;
 	static Time my_timestamp = 0;
@@ -491,7 +502,6 @@ int HandlePaging(
 	{
 		x = pev->xmotion.x_root;
 		y = pev->xmotion.y_root;
-		/* need to move the viewport */
 		if ((Scr.VxMax == 0 ||
 		     (x >= edge_thickness &&
 		      x < Scr.MyDisplayWidth  - edge_thickness)) &&
@@ -532,31 +542,28 @@ int HandlePaging(
 
 	do
 	{
-		if (FPending(dpy) > 0 &&
-		    (FCheckWindowEvent(
-			    dpy, Scr.PanFrameTop.win, LeaveWindowMask, pev) ||
-		     FCheckWindowEvent(
-			     dpy, Scr.PanFrameBottom.win, LeaveWindowMask,
-			     pev) ||
-		     FCheckWindowEvent(
-			     dpy, Scr.PanFrameLeft.win, LeaveWindowMask, pev) ||
-		     FCheckWindowEvent(
-			     dpy, Scr.PanFrameRight.win, LeaveWindowMask, pev)))
+		int rc;
+		Window JunkW;
+		int JunkC;
+		unsigned int JunkM;
+
+		if (FCheckPeekIfEvent(dpy, &e, test_button_event, NULL))
 		{
 			is_timestamp_valid = False;
 			add_time = 0;
 			return 0;
 		}
-		else if (FCheckMaskEvent(
-				 dpy, ButtonPressMask | ButtonReleaseMask, &e))
-		{
-			FPutBackEvent(dpy, &e);
-			is_timestamp_valid = False;
-			add_time = 0;
-			return 0;
-		}
 		/* get pointer location */
-		fev_get_evpos_or_query(dpy, Scr.Root, pev, &x, &y);
+		rc = FQueryPointer(
+			dpy, Scr.Root, &JunkW, &JunkW, &x, &y, &JunkC, &JunkC,
+			&JunkM);
+		if (rc == False)
+		{
+			/* pointer is on a different screen */
+			x = 0;
+			y = 0;
+		}
+
 		/* check actual pointer location since PanFrames can get buried
 		 * under window being moved or resized - mab */
 		if (x >= edge_thickness &&
@@ -566,14 +573,6 @@ int HandlePaging(
 		{
 			is_timestamp_valid = False;
 			add_time = 0;
-			discard_window_events(
-				Scr.PanFrameTop.win, LeaveWindowMask);
-			discard_window_events(
-				Scr.PanFrameBottom.win, LeaveWindowMask);
-			discard_window_events(
-				Scr.PanFrameLeft.win, LeaveWindowMask);
-			discard_window_events(
-				Scr.PanFrameRight.win, LeaveWindowMask);
 			return 0;
 		}
 		if (!fLoop && is_last_position_valid &&
@@ -607,6 +606,14 @@ int HandlePaging(
 		return 0;
 	}
 
+	/* Get the latest pointer position.  This is necessary as XFree 4.1.0.1
+	 * sometimes does not report mouse movement when it should. */
+	if (FQueryPointer(
+		    dpy, Scr.Root, &JunkRoot, &JunkChild, &x, &y, &JunkX,
+		    &JunkY, &JunkMask) == False)
+	{
+		/* pointer is on a different screen - ignore */
+	}
 	/* Move the viewport */
 	/* and/or move the cursor back to the approximate correct location */
 	/* that is, the same place on the virtual desktop that it */
@@ -822,19 +829,19 @@ void checkPanFrames(void)
 	/* correct the unmap variables if pan frame commands are set */
 	if (edge_thickness != 0)
 	{
-		if (Scr.PanFrameLeft.command != NULL)
+		if (Scr.PanFrameLeft.command != NULL || Scr.PanFrameLeft.command_leave != NULL)
 		{
 			do_unmap_l = False;
 		}
-		if (Scr.PanFrameRight.command != NULL)
+		if (Scr.PanFrameRight.command != NULL || Scr.PanFrameRight.command_leave != NULL)
 		{
 			do_unmap_r = False;
 		}
-		if (Scr.PanFrameBottom.command != NULL)
+		if (Scr.PanFrameBottom.command != NULL || Scr.PanFrameBottom.command_leave != NULL)
 		{
 			do_unmap_b = False;
 		}
-		if (Scr.PanFrameTop.command != NULL)
+		if (Scr.PanFrameTop.command != NULL || Scr.PanFrameTop.command_leave != NULL)
 		{
 			do_unmap_t = False;
 		}
@@ -953,21 +960,35 @@ void checkPanFrames(void)
  */
 void raisePanFrames(void)
 {
+	Window windows[4];
+	int n;
+
+	/* Note: make sure the stacking order of the pan frames is not changed
+	 * every time they are raised by using XRestackWindows. */
+	n = 0;
 	if (Scr.PanFrameTop.isMapped)
 	{
-		XRaiseWindow(dpy,Scr.PanFrameTop.win);
+		windows[n++] = Scr.PanFrameTop.win;
 	}
 	if (Scr.PanFrameLeft.isMapped)
 	{
-		XRaiseWindow(dpy,Scr.PanFrameLeft.win);
+		windows[n++] = Scr.PanFrameLeft.win;
 	}
 	if (Scr.PanFrameRight.isMapped)
 	{
-		XRaiseWindow(dpy,Scr.PanFrameRight.win);
+		windows[n++] = Scr.PanFrameRight.win;
 	}
 	if (Scr.PanFrameBottom.isMapped)
 	{
-		XRaiseWindow(dpy,Scr.PanFrameBottom.win);
+		windows[n++] = Scr.PanFrameBottom.win;
+	}
+	if (n > 0)
+	{
+		XRaiseWindow(dpy, windows[0]);
+		if (n > 1)
+		{
+			XRestackWindows(dpy, windows, n);
+		}
 	}
 
 	return;
@@ -1051,7 +1072,6 @@ void MoveViewport(int newx, int newy, Bool grab)
 	int PageTop, PageLeft;
 	int PageBottom, PageRight;
 	int txl, txr, tyt, tyb;
-	XEvent e;
 
 	if (grab)
 	{
@@ -1097,10 +1117,11 @@ void MoveViewport(int newx, int newy, Bool grab)
 	if (deltax || deltay)
 	{
 		BroadcastPacket(
-			M_NEW_PAGE, 7, Scr.Vx, Scr.Vy, Scr.CurrentDesk,
-			Scr.MyDisplayWidth, Scr.MyDisplayHeight,
-			(int)(Scr.VxMax / Scr.MyDisplayWidth) + 1,
-			(int)(Scr.VyMax / Scr.MyDisplayHeight) + 1);
+			M_NEW_PAGE, 7, (long)Scr.Vx, (long)Scr.Vy,
+			(long)Scr.CurrentDesk, (long)Scr.MyDisplayWidth,
+			(long)Scr.MyDisplayHeight,
+			(long)((Scr.VxMax / Scr.MyDisplayWidth) + 1),
+			(long)((Scr.VyMax / Scr.MyDisplayHeight) + 1));
 
 		/*
 		 * RBW - 11/13/1998      - new:  chase the chain
@@ -1109,7 +1130,7 @@ void MoveViewport(int newx, int newy, Bool grab)
 		 * of the stacking order up, to minimize the expose-redraw
 		 * overhead. Windows that will be moving into view will be
 		 * moved top down, for the same reason. Use the new
-		 * stacking-order chain, rather than the old last-focussed
+		 * stacking-order chain, rather than the old last-focused
 		 * chain.
 		 *
 		 * domivogt (29-Nov-1999): It's faster to first map windows
@@ -1121,18 +1142,18 @@ void MoveViewport(int newx, int newy, Bool grab)
 			/*
 			 * If the window is moving into the viewport...
 			 */
-			txl = t->frame_g.x;
-			tyt = t->frame_g.y;
-			txr = t->frame_g.x + t->frame_g.width - 1;
-			tyb = t->frame_g.y + t->frame_g.height - 1;
+			txl = t->g.frame.x;
+			tyt = t->g.frame.y;
+			txr = t->g.frame.x + t->g.frame.width - 1;
+			tyb = t->g.frame.y + t->g.frame.height - 1;
 			if (is_window_sticky_across_pages(t) &&
 			    !IS_VIEWPORT_MOVED(t))
 			{
 				/* the absolute position has changed */
-				t->normal_g.x -= deltax;
-				t->normal_g.y -= deltay;
-				t->max_g.x -= deltax;
-				t->max_g.y -= deltay;
+				t->g.normal.x -= deltax;
+				t->g.normal.y -= deltay;
+				t->g.max.x -= deltax;
+				t->g.max.y -= deltay;
 				/*  Block double move.  */
 				SET_VIEWPORT_MOVED(t, 1);
 			}
@@ -1157,10 +1178,10 @@ void MoveViewport(int newx, int newy, Bool grab)
 							t, False);
 					}
 					frame_setup_window(
-						t, t->frame_g.x + deltax,
-						t->frame_g.y + deltay,
-						t->frame_g.width,
-						t->frame_g.height, False);
+						t, t->g.frame.x + deltax,
+						t->g.frame.y + deltay,
+						t->g.frame.width,
+						t->g.frame.height, False);
 				}
 			}
 			/*  Bump to next win...  */
@@ -1173,10 +1194,10 @@ void MoveViewport(int newx, int newy, Bool grab)
 			 *If the window is not moving into the viewport...
 			 */
 			SET_VIEWPORT_MOVED(t, 1);
-			txl = t1->frame_g.x;
-			tyt = t1->frame_g.y;
-			txr = t1->frame_g.x + t1->frame_g.width - 1;
-			tyb = t1->frame_g.y + t1->frame_g.height - 1;
+			txl = t1->g.frame.x;
+			tyt = t1->g.frame.y;
+			txr = t1->g.frame.x + t1->g.frame.width - 1;
+			tyb = t1->g.frame.y + t1->g.frame.height - 1;
 			if (! (txr >= PageLeft && txl <= PageRight
 			       && tyb >= PageTop && tyt <= PageBottom)
 			    && !IS_VIEWPORT_MOVED(t1)
@@ -1196,10 +1217,10 @@ void MoveViewport(int newx, int newy, Bool grab)
 							t1, False);
 					}
 					frame_setup_window(
-						t1, t1->frame_g.x + deltax,
-						t1->frame_g.y + deltay,
-						t1->frame_g.width,
-						t1->frame_g.height, False);
+						t1, t1->g.frame.x + deltax,
+						t1->g.frame.y + deltay,
+						t1->g.frame.width,
+						t1->g.frame.height, False);
 				}
 			}
 			/*  Bump to next win...  */
@@ -1228,11 +1249,19 @@ void MoveViewport(int newx, int newy, Bool grab)
 	/* regrab buttons in case something got obscured or unobscured */
 	focus_grab_buttons_all();
 
+#if 0
+	/* dv (2004-07-01): I don't think that's a good idea.  We could eat too
+	 * many events. */
 	/* do this with PanFrames too ??? HEDU */
-	while (FCheckTypedEvent(dpy, MotionNotify, &e))
 	{
-		/* nothing */
+		XEvent e;
+
+		while (FCheckTypedEvent(dpy, MotionNotify, &e))
+		{
+			/* nothing */
+		}
 	}
+#endif
 	if (grab)
 	{
 		MyXUngrabServer(dpy);
@@ -1259,14 +1288,14 @@ void goto_desk(int desk)
 		Scr.CurrentDesk = desk;
 		MapDesk(desk, True);
 		focus_grab_buttons_all();
-		BroadcastPacket(M_NEW_DESK, 1, Scr.CurrentDesk);
+		BroadcastPacket(M_NEW_DESK, 1, (long)Scr.CurrentDesk);
 		/* FIXME: domivogt (22-Apr-2000): Fake a 'restack' for sticky
 		 * window upon desk change.  This is a workaround for a
 		 * problem in FvwmPager: The pager has a separate 'root'
 		 * window for each desk.  If the active desk changes, the
 		 * pager destroys sticky mini windows and creates new ones in
 		 * the other desktop 'root'.  But the pager can't know where to
-		 * stack them.  So we have to tell it ecplicitly where they
+		 * stack them.  So we have to tell it explicitly where they
 		 * go :-( This should be fixed in the pager, but right now the
 		 * pager doesn't maintain the stacking order. */
 		BroadcastRestackAllWindows();
@@ -1338,99 +1367,148 @@ Bool get_page_arguments(char *action, int *page_x, int *page_y)
 	int suffix[2];
 	char *token;
 	char *taction;
-	Bool xwrap = False;
-	Bool ywrap = False;
+	int wrapx;
+	int wrapy;
+	int limitdeskx;
+	int limitdesky;
+	int startx;
+	int starty;
 
-	taction = GetNextToken(action, &token);
-	if (token == NULL)
+	wrapx = 0;
+	wrapy = 0;
+	limitdeskx = 1;
+	limitdesky = 1;
+	startx = *page_x;
+	starty = *page_y;
+	for (; ; action = taction)
 	{
-		*page_x = Scr.Vx;
-		*page_y = Scr.Vy;
-		return True;
-	}
-	if (StrEquals(token, "prev"))
-	{
-		/* last page selected */
-		*page_x = prev_page_x;
-		*page_y = prev_page_y;
-		free(token);
-		return True;
-	}
-	if (StrEquals(token, "xwrap"))
-	{
-		xwrap = True;
-	}
-	else if (StrEquals(token, "ywrap"))
-	{
-		ywrap = True;
-	}
-	else if (StrEquals(token, "xywrap"))
-	{
-		xwrap = True;
-		ywrap = True;
-	}
-	free(token);
-	if (xwrap || ywrap)
-	{
-		GetNextToken(taction, &token);
+		int do_reverse;
+
+		token = PeekToken(action, &taction);
 		if (token == NULL)
 		{
 			*page_x = Scr.Vx;
 			*page_y = Scr.Vy;
 			return True;
 		}
-		action = taction;
-		free(token);
+		if (StrEquals(token, "prev"))
+		{
+			/* last page selected */
+			*page_x = prev_page_x;
+			*page_y = prev_page_y;
+			return True;
+		}
+		do_reverse = 0;
+		for ( ; *token == '!'; token++)
+		{
+			do_reverse = !do_reverse;
+		}
+		if (StrEquals(token, "wrapx"))
+		{
+			wrapx = (1 ^ do_reverse);
+		}
+		else if (StrEquals(token, "wrapy"))
+		{
+			wrapy = (1 ^ do_reverse);
+		}
+		else if (StrEquals(token, "nodesklimitx"))
+		{
+			limitdeskx = (0 ^ do_reverse);
+		}
+		else if (StrEquals(token, "nodesklimitx"))
+		{
+			limitdesky = (0 ^ do_reverse);
+		}
+		else
+		{
+			/* no more options */
+			break;
+		}
 	}
 
-	if (GetSuffixedIntegerArguments(action, NULL, val, 2, "p", suffix) !=
+	if (GetSuffixedIntegerArguments(action, NULL, val, 2, "pw", suffix) !=
 	    2)
 	{
 		return 0;
 	}
 
-	if (val[0] >= 0 || suffix[0] == 1)
+	if (suffix[0] == 1)
+	{
+		*page_x = val[0] * Scr.MyDisplayWidth + Scr.Vx;
+	}
+	else if (suffix[0] == 2)
+	{
+		*page_x += val[0] * Scr.MyDisplayWidth;
+	}
+	else if (val[0] >= 0)
 	{
 		*page_x = val[0] * Scr.MyDisplayWidth;
 	}
 	else
 	{
-		*page_x = (val[0]+1) * Scr.MyDisplayWidth + Scr.VxMax;
+		*page_x = (val[0] + 1) * Scr.MyDisplayWidth + Scr.VxMax;
 	}
-	if (val[1] >= 0 || suffix[1] == 1)
+	if (suffix[1] == 1)
+	{
+		*page_y = val[1] * Scr.MyDisplayHeight + Scr.Vy;
+	}
+	else if (suffix[1] == 2)
+	{
+		*page_y += val[1] * Scr.MyDisplayHeight;
+	}
+	else if (val[1] >= 0)
 	{
 		*page_y = val[1] * Scr.MyDisplayHeight;
 	}
 	else
 	{
-		*page_y = (val[1]+1) * Scr.MyDisplayHeight + Scr.VyMax;
-	}
-	/* handle 'p' suffix */
-	if (suffix[0] == 1)
-	{
-		*page_x += Scr.Vx;
-	}
-	if (suffix[1] == 1)
-	{
-		*page_y += Scr.Vy;
+		*page_y = (val[1] + 1) * Scr.MyDisplayHeight + Scr.VyMax;
 	}
 
 	/* limit to desktop size */
-	if (*page_x < 0)
+	if (limitdeskx && !wrapx)
 	{
-		*page_x = 0;
+		if (*page_x < 0)
+		{
+			*page_x = 0;
+		}
+		else if (*page_x > Scr.VxMax)
+		{
+			*page_x = Scr.VxMax;
+		}
 	}
-	if (*page_y < 0)
+	else if (limitdeskx && wrapx)
 	{
-		*page_y = 0;
+		while (*page_x < 0)
+		{
+			*page_x += Scr.VxMax + Scr.MyDisplayWidth;
+		}
+		while (*page_x > Scr.VxMax)
+		{
+			*page_x -= Scr.VxMax + Scr.MyDisplayWidth;
+		}
 	}
-	if (*page_x > Scr.VxMax)
+	if (limitdesky && !wrapy)
 	{
-		*page_x = Scr.VxMax;
+		if (*page_y < 0)
+		{
+			*page_y = 0;
+		}
+		else if (*page_y > Scr.VyMax)
+		{
+			*page_y = Scr.VyMax;
+		}
 	}
-	if (*page_y > Scr.VyMax)
+	else if (limitdesky && wrapy)
 	{
-		*page_y = Scr.VyMax;
+		while (*page_y < 0)
+		{
+			*page_y += Scr.VyMax + Scr.MyDisplayHeight;
+		}
+		while (*page_y > Scr.VyMax)
+		{
+			*page_y -= Scr.VyMax + Scr.MyDisplayHeight;
+		}
 	}
 
 	return True;
@@ -1560,6 +1638,111 @@ void CMD_EdgeCommand(F_CMD_ARGS)
 	return;
 }
 
+/* EdgeLeaveCommand - binds a function to a pan frame Leave event */
+void CMD_EdgeLeaveCommand(F_CMD_ARGS)
+{
+	direction_t direction;
+	char * command;
+
+	/* get the direction */
+	direction = gravity_parse_dir_argument(action, &action, DIR_NONE);
+
+	if (direction >= 0 && direction <= DIR_MAJOR_MASK)
+	{
+
+		/* check if the command does contain at least one token */
+		command = safestrdup(action);
+		if (PeekToken(action , &action) == NULL)
+		{
+			/* the command does not contain a token so
+			   the command of this edge is removed */
+			free(command);
+			command = NULL;
+		}
+		/* assign command to the edge(s) */
+		if (direction == DIR_N)
+		{
+			if (Scr.PanFrameTop.command_leave != NULL)
+			{
+				free(Scr.PanFrameTop.command_leave);
+			}
+			Scr.PanFrameTop.command_leave = command;
+		}
+		else if (direction == DIR_S)
+		{
+			if (Scr.PanFrameBottom.command_leave != NULL)
+			{
+				free(Scr.PanFrameBottom.command_leave);
+			}
+			Scr.PanFrameBottom.command_leave = command;
+		}
+		else if (direction == DIR_W)
+		{
+			if (Scr.PanFrameLeft.command_leave != NULL)
+			{
+				free(Scr.PanFrameLeft.command_leave);
+			}
+			Scr.PanFrameLeft.command_leave = command;
+		}
+		else if (direction == DIR_E)
+		{
+			if (Scr.PanFrameRight.command_leave != NULL)
+			{
+				free(Scr.PanFrameRight.command_leave);
+			}
+			Scr.PanFrameRight.command_leave = command;
+		}
+		else
+		{
+			/* this should never happen */
+			fvwm_msg(ERR, "EdgeLeaveCommand",
+				 "Internal error in CMD_EdgeLeaveCommand");
+		}
+
+	}
+	else
+	{
+
+		/* check if the argument does contain at least one token */
+		if (PeekToken(action , &action) == NULL)
+		{
+			/* Just plain EdgeLeaveCommand, so all edge commands are
+			 * removed */
+			if (Scr.PanFrameTop.command_leave != NULL)
+			{
+				free(Scr.PanFrameTop.command_leave);
+				Scr.PanFrameTop.command_leave = NULL;
+			}
+			if (Scr.PanFrameBottom.command_leave != NULL)
+			{
+				free(Scr.PanFrameBottom.command_leave);
+				Scr.PanFrameBottom.command_leave = NULL;
+			}
+			if (Scr.PanFrameLeft.command_leave != NULL)
+			{
+				free(Scr.PanFrameLeft.command_leave);
+				Scr.PanFrameLeft.command_leave = NULL;
+			}
+			if (Scr.PanFrameRight.command_leave != NULL)
+			{
+				free(Scr.PanFrameRight.command_leave);
+				Scr.PanFrameRight.command_leave = NULL;
+			}
+		}
+		else
+		{
+			/* not a proper direction */
+			fvwm_msg(ERR, "EdgeLeaveCommand",
+				 "EdgeLeaveCommand [direction [function]]");
+		}
+	}
+
+	/* maybe something has changed so we adapt the pan frames */
+	checkPanFrames();
+
+	return;
+}
+
 void CMD_EdgeThickness(F_CMD_ARGS)
 {
 	int val, n;
@@ -1589,6 +1772,7 @@ void CMD_EdgeThickness(F_CMD_ARGS)
 void CMD_EdgeScroll(F_CMD_ARGS)
 {
 	int val1, val2, val1_unit, val2_unit, n;
+	char *token;
 
 	n = GetTwoArguments(action, &val1, &val2, &val1_unit, &val2_unit);
 	if (n != 2)
@@ -1603,7 +1787,7 @@ void CMD_EdgeScroll(F_CMD_ARGS)
 	 * if edgescroll >1000 and <100000
 	 * wrap at edges of desktop (a "spherical" desktop)
 	 */
-	if (val1 >= 1000)
+	if (val1 >= 1000 && val1_unit != 100)
 	{
 		val1 /= 1000;
 		Scr.flags.do_edge_wrap_x = 1;
@@ -1612,7 +1796,7 @@ void CMD_EdgeScroll(F_CMD_ARGS)
 	{
 		Scr.flags.do_edge_wrap_x = 0;
 	}
-	if (val2 >= 1000)
+	if (val2 >= 1000 && val2_unit != 100)
 	{
 		val2 /= 1000;
 		Scr.flags.do_edge_wrap_y = 1;
@@ -1622,6 +1806,26 @@ void CMD_EdgeScroll(F_CMD_ARGS)
 		Scr.flags.do_edge_wrap_y = 0;
 	}
 
+	action=SkipNTokens(action,2);
+	token = PeekToken(action, NULL);
+
+	if (token)
+	{
+		if (StrEquals(token, "wrap"))
+		{
+			Scr.flags.do_edge_wrap_x = 1;
+			Scr.flags.do_edge_wrap_y = 1;
+		}
+		else if (StrEquals(token, "wrapx"))
+		{
+			Scr.flags.do_edge_wrap_x = 1;
+		}
+		else if (StrEquals(token, "wrapy"))
+		{
+			Scr.flags.do_edge_wrap_y = 1;
+		}
+	}
+
 	Scr.EdgeScrollX = val1 * val1_unit / 100;
 	Scr.EdgeScrollY = val2 * val2_unit / 100;
 
@@ -1765,10 +1969,11 @@ void CMD_DesktopSize(F_CMD_ARGS)
 	Scr.VyMax = (val[1] <= 0) ?
 		0: val[1]*Scr.MyDisplayHeight-Scr.MyDisplayHeight;
 	BroadcastPacket(
-		M_NEW_PAGE, 7, Scr.Vx, Scr.Vy, Scr.CurrentDesk,
-		Scr.MyDisplayWidth, Scr.MyDisplayHeight,
-		(int)(Scr.VxMax / Scr.MyDisplayWidth) + 1,
-		(int)(Scr.VyMax / Scr.MyDisplayHeight) + 1);
+		M_NEW_PAGE, 7, (long)Scr.Vx, (long)Scr.Vy,
+		(long)Scr.CurrentDesk, (long)Scr.MyDisplayWidth,
+		(long)Scr.MyDisplayHeight,
+		(long)((Scr.VxMax / Scr.MyDisplayWidth) + 1),
+		(long)((Scr.VyMax / Scr.MyDisplayHeight) + 1));
 
 	checkPanFrames();
 	/* update GNOME pager */
@@ -1785,7 +1990,7 @@ void CMD_DesktopSize(F_CMD_ARGS)
  */
 void CMD_GotoDesk(F_CMD_ARGS)
 {
-	goto_desk(GetDeskNumber(action));
+	goto_desk(GetDeskNumber(action, Scr.CurrentDesk));
 
 	return;
 }
@@ -1842,7 +2047,7 @@ void CMD_GotoDeskAndPage(F_CMD_ARGS)
 		Scr.CurrentDesk = val[0];
 		MapDesk(val[0], True);
 		focus_grab_buttons_all();
-		BroadcastPacket(M_NEW_DESK, 1, Scr.CurrentDesk);
+		BroadcastPacket(M_NEW_DESK, 1, (long)Scr.CurrentDesk);
 		/* FIXME: domivogt (22-Apr-2000): Fake a 'restack' for sticky
 		 * window upon desk change.  This is a workaround for a
 		 * problem in FvwmPager: The pager has a separate 'root'
@@ -1856,7 +2061,7 @@ void CMD_GotoDeskAndPage(F_CMD_ARGS)
 	}
 	else
 	{
-		BroadcastPacket(M_NEW_DESK, 1, Scr.CurrentDesk);
+		BroadcastPacket(M_NEW_DESK, 1, (long)Scr.CurrentDesk);
 	}
 	EWMH_SetCurrentDesktop();
 	GNOME_SetCurrentDesk();
@@ -1870,6 +2075,8 @@ void CMD_GotoPage(F_CMD_ARGS)
 	int x;
 	int y;
 
+	x = Scr.Vx;
+	y = Scr.Vy;
 	if (!get_page_arguments(action, &x, &y))
 	{
 		fvwm_msg(
@@ -1877,7 +2084,22 @@ void CMD_GotoPage(F_CMD_ARGS)
 			"GotoPage: invalid arguments: %s", action);
 		return;
 	}
-
+	if (x < 0)
+	{
+		x = 0;
+	}
+	if (x > Scr.VxMax)
+	{
+		x = Scr.VxMax;
+	}
+	if (y < 0)
+	{
+		y = 0;
+	}
+	if (y > Scr.VyMax)
+	{
+		y = Scr.VyMax;
+	}
 	MoveViewport(x,y,True);
 
 	return;
@@ -1889,7 +2111,7 @@ void CMD_MoveToDesk(F_CMD_ARGS)
 	int desk;
 	FvwmWindow * const fw = exc->w.fw;
 
-	desk = GetDeskNumber(action);
+	desk = GetDeskNumber(action, fw->Desk);
 	if (desk == fw->Desk)
 	{
 		return;
