@@ -40,7 +40,12 @@
 #include "functions.h"
 #include "misc.h"
 #include "screen.h"
+#include "menudim.h"
+#include "menuitem.h"
+#include "menuroot.h"
+#include "menustyle.h"
 #include "menus.h"
+#include "menuparameters.h"
 #include "conditional.h"
 #include "stack.h"
 #include "focus.h"
@@ -193,8 +198,8 @@ static int compareReverse(const  FvwmWindow **a, const  FvwmWindow **b)
  * title directly. */
 void CMD_WindowList(F_CMD_ARGS)
 {
-	MenuRoot *mr;
-	MenuParameters mp;
+	struct MenuRoot *mr;
+	struct MenuParameters mp;
 	char* ret_action = NULL;
 	FvwmWindow *t;
 	FvwmWindow **windowList;
@@ -916,11 +921,11 @@ void CMD_WindowList(F_CMD_ARGS)
 					int scr;
 
 					fscr.xypos.x =
-						Scr.Vx + t->frame_g.x +
-						t->frame_g.width / 2;
+						Scr.Vx + t->g.frame.x +
+						t->g.frame.width / 2;
 					fscr.xypos.y =
-						Scr.Vy + t->frame_g.y +
-						t->frame_g.height / 2;
+						Scr.Vy + t->g.frame.y +
+						t->g.frame.height / 2;
 					scr = FScreenGetScrId(
 						&fscr, FSCREEN_XYPOS);
 					sprintf(loc, "@%d", scr);
@@ -929,16 +934,16 @@ void CMD_WindowList(F_CMD_ARGS)
 				if (flags & SHOW_PAGE_X)
 				{
 					sprintf(loc, "+%d",
-						(Scr.Vx + t->frame_g.x +
-						 t->frame_g.width / 2) /
+						(Scr.Vx + t->g.frame.x +
+						 t->g.frame.width / 2) /
 						Scr.MyDisplayWidth);
 					strcat(tname, loc);
 				}
 				if (flags & SHOW_PAGE_Y)
 				{
 					sprintf(loc, "+%d",
-						(Scr.Vy + t->frame_g.y +
-						 t->frame_g.height/2) /
+						(Scr.Vy + t->g.frame.y +
+						 t->g.frame.height/2) /
 						Scr.MyDisplayHeight);
 					strcat(tname, loc);
 				}
@@ -950,9 +955,9 @@ void CMD_WindowList(F_CMD_ARGS)
 				}
 				strcat(tname, ":");
 				get_window_borders(t, &b);
-				dheight = t->frame_g.height -
+				dheight = t->g.frame.height -
 					b.total_size.height;
-				dwidth = t->frame_g.width -
+				dwidth = t->g.frame.width -
 					b.total_size.width;
 				dwidth -= t->hints.base_width;
 				dheight -= t->hints.base_height;
@@ -963,22 +968,22 @@ void CMD_WindowList(F_CMD_ARGS)
 				strcat(tname, loc);
 				sprintf(loc,"x%d",dheight);
 				strcat(tname, loc);
-				if (t->frame_g.x >=0)
+				if (t->g.frame.x >=0)
 				{
-					sprintf(loc,"+%d",t->frame_g.x);
+					sprintf(loc,"+%d",t->g.frame.x);
 				}
 				else
 				{
-					sprintf(loc,"%d",t->frame_g.x);
+					sprintf(loc,"%d",t->g.frame.x);
 				}
 				strcat(tname, loc);
-				if (t->frame_g.y >=0)
+				if (t->g.frame.y >=0)
 				{
-					sprintf(loc,"+%d",t->frame_g.y);
+					sprintf(loc,"+%d",t->g.frame.y);
 				}
 				else
 				{
-					sprintf(loc,"%d",t->frame_g.y);
+					sprintf(loc,"%d",t->g.frame.y);
 				}
 				strcat(tname, loc);
 
