@@ -1,7 +1,6 @@
 /* -*-c-*- */
-/* Copyright (C) 2002  Olivier Chapuis
- *
- * This program is free software; you can redistribute it and/or modify
+/* Copyright (C) 2002  Olivier Chapuis */
+/* This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
@@ -16,7 +15,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* ---------------------------- included header files ----------------------- */
+/* ---------------------------- included header files ---------------------- */
 
 #include "config.h"
 
@@ -33,25 +32,25 @@
 #include "PictureUtils.h"
 #include "FImage.h"
 
-/* ---------------------------- local definitions --------------------------- */
+/* ---------------------------- local definitions -------------------------- */
 
-/* ---------------------------- local macros -------------------------------- */
+/* ---------------------------- local macros ------------------------------- */
 
-/* ---------------------------- imports ------------------------------------- */
+/* ---------------------------- imports ------------------------------------ */
 
-/* ---------------------------- included code files ------------------------- */
+/* ---------------------------- included code files ------------------------ */
 
-/* ---------------------------- local types --------------------------------- */
+/* ---------------------------- local types -------------------------------- */
 
-/* ---------------------------- forward declarations ------------------------ */
+/* ---------------------------- forward declarations ----------------------- */
 
-/* ---------------------------- local variables ----------------------------- */
+/* ---------------------------- local variables ---------------------------- */
 
 static Bool PGrabImageError = True;
 
-/* ---------------------------- exported variables (globals) ---------------- */
+/* ---------------------------- exported variables (globals) --------------- */
 
-/* ---------------------------- local functions ----------------------------- */
+/* ---------------------------- local functions ---------------------------- */
 static
 int FSetBackingStore(Display *dpy, Window win, int backing_store)
 {
@@ -918,7 +917,7 @@ Pixmap PCreateDitherPixmap(
 	return out_pix;
 }
 
-/* ---------------------------- interface functions ------------------------- */
+/* ---------------------------- interface functions ------------------------ */
 
 Pixmap PictureBitmapToPixmap(
 	Display *dpy, Window win, Pixmap src, int depth, GC gc,
@@ -957,6 +956,7 @@ void PGraphicsRenderPixmaps(
 	t_fra.added_alpha_percent = 100;
 	t_fra.tint_percent = 0;
 	t_fra.mask = 0;
+	t_fra.tint = None;
 
 	if (fra)
 	{
@@ -1192,7 +1192,7 @@ FvwmPicture *PGraphicsCreateTiledPicture(
 	return q;
 }
 
-Pixmap PGraphicsCreateTransprency(
+Pixmap PGraphicsCreateTransparency(
 	Display *dpy, Window win, FvwmRenderAttributes *fra, GC gc,
 	int x, int y, int width, int height, Bool parent_relative)
 {
@@ -1213,7 +1213,7 @@ Pixmap PGraphicsCreateTransprency(
 
 	if (parent_relative)
 	{
-		/* this block is not usefull if backing store ... */
+		/* this block is not useful if backing store ... */
 		if (!XGetGeometry(
 			dpy, win, &root, (int *)&junk, (int *)&junk,
 			(unsigned int *)&sw, (unsigned int *)&sh,
@@ -1326,7 +1326,7 @@ void PGraphicsTintRectangle(
 		fra.tint = tint;
 		fra.tint_percent = tint_percent;
 		fra.mask = FRAM_DEST_IS_A_WINDOW | FRAM_HAVE_TINT;
-		p = PGraphicsCreateTransprency(
+		p = PGraphicsCreateTransparency(
 			dpy, dest, &fra, gc, dest_x, dest_y, dest_w, dest_h,
 			False);
 		if (p)
@@ -1339,13 +1339,32 @@ void PGraphicsTintRectangle(
 	}
 }
 
-#if 0 /* humm... maybe usefull one day with menus */
+#if 0 /* humm... maybe useful one day with menus */
 Pixmap PGraphicsCreateTranslucent(
 	Display *dpy, Window win, FvwmRenderAttributes *fra, GC gc,
 	int x, int y, int width, int height)
 {
 	Pixmap r = None;
 	int gx = x, gy = y, gh = height, gw = width;
+	FvwmRenderAttributes t_fra;
+	Pixmap root_pix = None;
+	Pixmap dp = None;
+	int dummy;
+
+	t_fra.added_alpha_percent = 100;
+	t_fra.tint_percent = 0;
+	t_fra.tint = 0;
+	t_fra.mask = 0;
+
+	if (fra)
+	{
+		if (fra->mask & FRAM_HAVE_TINT)
+		{
+			t_fra.tint_percent = fra->tint_percent;
+			t_fra.tint = fra->tint;
+			t_fra.mask = FRAM_HAVE_TINT;
+		}
+	}
 
 	if (x >= DisplayWidth(dpy, DefaultScreen(dpy)))
 	{
@@ -1381,27 +1400,52 @@ Pixmap PGraphicsCreateTranslucent(
 	{
 		gh = DisplayHeight(dpy, DefaultScreen(dpy)) - gy;
 	}
+	{
+		/* make a screen shoot */
+		GC my_gc;
+		unsigned long valuemask = GCSubwindowMode;
+		XGCValues values;
+
+		values.subwindow_mode = IncludeInferiors;
+		root_pix = XCreatePixmap(dpy, win, gw, gh, Pdepth);
+		my_gc = fvwmlib_XCreateGC(dpy, win, 0, NULL);
+		XChangeGC(dpy, my_gc, valuemask, &values);
+		XGrabServer(dpy);
+		XCopyArea(
+			dpy, DefaultRootWindow(dpy), root_pix, my_gc,
+			gx, gy, gw, gh, 0, 0);
+		XUngrabServer(dpy);
+		XFreeGC(dpy,my_gc);
+	}
 	if (XRenderSupport && FRenderGetExtensionSupported())
 	{
-		r = XCreatePixmap(dpy, win, width, height, Pdepth);
+		r = XCreatePixmap(dpy, win, gw, gh, Pdepth);
 		if (FRenderRender(
-			dpy, win, DefaultRootWindow(dpy), None, None, Pdepth,
-			100, 0, 50, r, gc, None,
-			gx, gy, gw, gh, 0, 0, gw, gh, False))
+			dpy, win, root_pix, None, None, Pdepth,
+			t_fra.added_alpha_percent, t_fra.tint,
+			t_fra.tint_percent, r, gc, None,
+			0, 0, gw, gh, 0, 0, gw, gh, False))
 		{
 			goto bail;
 		}
 		XFreePixmap(dpy, r);
+		r = None;
 	}
-#if 0
 	r = PCreateRenderPixmap(
-		dpy, win, ParentRelative, None, None, Pdepth, 100, fra->tint,
-		fra->tint_percent,
-		True, win,
-		gc, None, None, gx, gy, gw, gh, gx, gy, gw, gh,
+		dpy, win, root_pix, None, None, Pdepth, 100,
+		fra->tint, fra->tint_percent, True, win,
+		gc, None, None, 0, 0, gw, gh, gx, gy, gw, gh,
 		False, &dummy, &dummy, &dummy, &dp);
-#endif
+
  bail:
+	if (root_pix)
+	{
+		XFreePixmap(dpy, root_pix);
+	}
+	if (dp)
+	{
+		XFreePixmap(dpy, dp);
+	}
 	return r;
 }
 #endif
