@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /* FvwmIconBox Module --- Copyright 1994, 1995 Nobutaka Suzuki.
  *
  * No guarantees or warantees or anything are provided or implied in
@@ -43,7 +44,6 @@
 
 #include <unistd.h>
 #include <ctype.h>
-#include <stdlib.h>
 #include "libs/Module.h"
 #include "libs/fvwmsignal.h"
 
@@ -68,7 +68,6 @@
 
 #include "FvwmIconBox.h"
 
-
 char *MyName;
 
 Display *dpy;                   /* which display are we talking to */
@@ -90,6 +89,9 @@ int Iconcolorset = -1;
 int IconHicolorset = -1;
 Bool UseSkipList = False;
 Bool Swallowed = False;
+Window SwallowerWin = False;
+Bool Shaded = False;
+Bool SwallowerShaded = False;
 
 /* same strings as in misc.c */
 char NoClass[] = "NoClass";
@@ -153,6 +155,15 @@ int icon_relief = 4;
 int margin1 = 8;
 int margin2 = 6;
 
+/* true if iconified windows doesn't have () around the title */
+Bool iconified_has_normal_title = False;
+
+int normal_title_relief = 2;
+int iconified_title_relief = 2;
+
+Bool normal_title_inverted_relief = False;
+Bool iconified_title_inverted_relief = False;
+
 Pixmap IconwinPixmap = None;
 char *IconwinPixmapFile = NULL;
 
@@ -181,7 +192,6 @@ int sortby = UNSORT;
 
 char* AnimCommand = NULL;
 
-int save_color_limit = 0;                   /* color limit from config */
 static Bool have_double_click = False;
 static Bool is_dead_pipe = False;
 
@@ -193,11 +203,11 @@ static void change_colorset(int color);
 Bool do_allow_bad_access = False;
 Bool was_bad_access = False;
 
-/************************************************************************
+/*
   Main
   Based on main() from GoodStuff:
 	Copyright 1993, Robert Nation.
-************************************************************************/
+*/
 int main(int argc, char **argv)
 {
   char *display_name = NULL;
@@ -217,7 +227,7 @@ int main(int argc, char **argv)
     exit(1);
   }
 
-  /* alise support */
+  /* aliases support */
   if (argc == 7 && argv[6] != NULL)
   {
     MyName = safemalloc(strlen(argv[6])+1);
@@ -327,11 +337,11 @@ int main(int argc, char **argv)
 
 
 
-/************************************************************************
+/*
   Loop
   Based on Loop() from GoodStuff:
       Copyright 1993, Robert Nation.
-************************************************************************/
+*/
 void Loop(void)
 {
   Window root;
@@ -365,7 +375,7 @@ void Loop(void)
 		Event.xexpose.x = ex;
 		Event.xexpose.y = ey;
 		Event.xexpose.width = ex2 - ex;
-		Event.xexpose.height = ey2 - ey;  
+		Event.xexpose.height = ey2 - ey;
 		if (Event.xany.window == main_win)
 		{
 			RedrawWindow(&Event);
@@ -696,11 +706,11 @@ struct icon_info *Search(Window w)
   return NULL;
 }
 
-/************************************************************************
+/*
  *
  * Draw the window
  *
- ***********************************************************************/
+ */
 void RedrawWindow(XEvent *evp)
 {
   XEvent dummy;
@@ -760,6 +770,7 @@ void RedrawIcon(struct icon_info *item, int f, XEvent *evp)
 {
 	int hr, len;
 	int diff, lm ,w, h, tw;
+	int title_relief;
 	char label[256];
 	int cs;
 	XRectangle inter;
@@ -794,6 +805,7 @@ void RedrawIcon(struct icon_info *item, int f, XEvent *evp)
 	else
 	{
 		cs = Iconcolorset;
+		XSetWindowBackground(dpy, item->IconWin, icon_back_pix);
 	}
 
 	/* icon pixmap */
@@ -806,7 +818,7 @@ void RedrawIcon(struct icon_info *item, int f, XEvent *evp)
 			if (evp)
 			{
 				if (!frect_get_intersection(
-					evp->xexpose.x, evp->xexpose.y, 
+					evp->xexpose.x, evp->xexpose.y,
 					evp->xexpose.width, evp->xexpose.height,
 					hr, hr, item->icon_w, item->icon_h,
 					&inter))
@@ -842,7 +854,7 @@ void RedrawIcon(struct icon_info *item, int f, XEvent *evp)
 						XClearArea(
 							dpy, item->icon_pixmap_w,
 							evp->xexpose.x,
-							evp->xexpose.y, 
+							evp->xexpose.y,
 							evp->xexpose.width,
 							evp->xexpose.height,
 							False);
@@ -885,17 +897,16 @@ void RedrawIcon(struct icon_info *item, int f, XEvent *evp)
 		{
 			if (item->icon_w > 0 && item->icon_h > 0)
 				RelieveRectangle(dpy, item->icon_pixmap_w,
-						 0, 0, item->icon_w
-						 +icon_relief - 1,
+						 0, 0,
+						 item->icon_w + icon_relief - 1,
 						 item->icon_h + icon_relief - 1,
 						 IconReliefGC,
 						 IconShadowGC, 2);
 			else
 				RelieveRectangle(dpy, item->icon_pixmap_w,
-						 0, 0, max_icon_width
-						 + icon_relief - 1,
-						 max_icon_height +
-						 icon_relief - 1,
+						 0, 0,
+						 max_icon_width + icon_relief - 1,
+						 max_icon_height + icon_relief - 1,
 						 IconReliefGC,
 						 IconShadowGC, 2);
 		}
@@ -909,12 +920,23 @@ void RedrawIcon(struct icon_info *item, int f, XEvent *evp)
 
 		if (IS_ICONIFIED(item))
 		{
-			sprintf(label, "(%s)", item->name);
+		        if( ! iconified_has_normal_title )
+			        sprintf(label, "(%s)", item->name);
+			else
+			        strcpy(label, item->name);
+
 		}
 		else
 		{
 			strcpy(label, item->name);
 		}
+
+		/* calculate the actual relief thickness */
+		title_relief = IS_ICONIFIED(item) ?
+			                iconified_title_relief :
+			                normal_title_relief;
+
+
 		if (evp)
 		{
 			inter.x = evp->xexpose.x;
@@ -934,13 +956,15 @@ void RedrawIcon(struct icon_info *item, int f, XEvent *evp)
 		tw = FlocaleTextWidth(Ffont, label, len);
 		diff = max_icon_width + icon_relief - tw;
 		lm = diff/2;
-		lm = lm > 4 ? lm : 4;
+		lm = lm > 2 + title_relief ? lm : 2 + title_relief;
 
 		FwinString->str = label;
 		FwinString->win = item->IconWin;
 		FwinString->gc =  NormalGC;
 		FwinString->x = lm;
-		FwinString->y = 3 + Ffont->ascent;
+		FwinString->y = 1 + max(normal_title_relief,
+					iconified_title_relief) +
+		                Ffont->ascent;
 		if (cs >= 0)
 		{
 			FwinString->colorset = &Colorset[cs];
@@ -951,18 +975,24 @@ void RedrawIcon(struct icon_info *item, int f, XEvent *evp)
 			FwinString->flags.has_colorset = False;
 		}
 
+
+
 		if (Hilite == item)
 		{
 			XRaiseWindow(dpy, item->IconWin);
 			XMoveResizeWindow(dpy, item->IconWin,
 					  item->x + min(0, (diff - 8))/2,
 					  item->y + h,
-					  max(tw + 8, w), 6 + Ffont->height);
+					  max(tw + 4 + 2 * title_relief, w),
+					  2 +
+					  2 * max(iconified_title_relief,
+						  normal_title_relief) +
+					  Ffont->height);
 			if (evp)
 			{
 				XClearArea(
 					dpy, item->IconWin,
-					evp->xexpose.x, evp->xexpose.y, 
+					evp->xexpose.x, evp->xexpose.y,
 					evp->xexpose.width, evp->xexpose.height,
 					False);
 			}
@@ -971,21 +1001,54 @@ void RedrawIcon(struct icon_info *item, int f, XEvent *evp)
 				XClearWindow(dpy, item->IconWin);
 			}
 			FlocaleDrawString(dpy, Ffont, FwinString, 0);
-			RelieveRectangle(dpy, item->IconWin, 0, 0,
-					 max(tw + 8, w) - 1,
-					 6 + Ffont->height - 1,
-					 IconReliefGC, IconShadowGC, 2);
+
+			if((IS_ICONIFIED(item) &&
+			    !iconified_title_inverted_relief) ||
+			   (!IS_ICONIFIED(item) &&
+			    !normal_title_inverted_relief))
+			{
+			        RelieveRectangle(dpy, item->IconWin, 0, 0,
+						 max(tw + 4 + 2 * title_relief,
+						     w)
+						 - 1,
+						 2 +
+						 2 *
+						 max(iconified_title_relief,
+						     normal_title_relief) +
+						 Ffont->height - 1,
+						 IconReliefGC, IconShadowGC,
+						 title_relief);
+			}
+			else
+			{
+			         RelieveRectangle(dpy, item->IconWin, 0, 0,
+						 max(tw + 4 + 2 * title_relief,
+						     w)
+						 - 1,
+						 2 +
+						 2 *
+						 max(iconified_title_relief,
+						     normal_title_relief) +
+						 Ffont->height - 1,
+						 IconShadowGC, IconReliefGC,
+						 title_relief);
+			}
 		}
+
 		else
 		{
 			XMoveResizeWindow(dpy, item->IconWin,
 					  item->x, item->y + h,
-					  w, 6 + Ffont->height);
+					  w,
+					  2 +
+					  2 * max(iconified_title_relief,
+						  normal_title_relief) +
+					  Ffont->height);
 			if (evp)
 			{
 				XClearArea(
 					dpy, item->IconWin,
-					evp->xexpose.x, evp->xexpose.y, 
+					evp->xexpose.x, evp->xexpose.y,
 					evp->xexpose.width, evp->xexpose.height,
 					False);
 			}
@@ -994,9 +1057,35 @@ void RedrawIcon(struct icon_info *item, int f, XEvent *evp)
 				XClearWindow(dpy, item->IconWin);
 			}
 			FlocaleDrawString(dpy, Ffont, FwinString, 0);
-			RelieveRectangle(dpy, item->IconWin, 0, 0,
-					 w - 1, 5 + Ffont->height,
-					 IconReliefGC, IconShadowGC, 2);
+
+			if((IS_ICONIFIED(item) &&
+			    !iconified_title_inverted_relief) ||
+			   (!IS_ICONIFIED(item) &&
+			    !normal_title_inverted_relief))
+			{
+			        RelieveRectangle(dpy, item->IconWin, 0, 0,
+						 w - 1,
+						 2 +
+						 2 *
+						 max(iconified_title_relief,
+						     normal_title_relief) +
+						 Ffont->height - 1,
+						 IconReliefGC, IconShadowGC,
+						 title_relief );
+			}
+			else
+			{
+			        RelieveRectangle(dpy, item->IconWin, 0, 0,
+						 w - 1,
+						 2 +
+						 2 *
+						 max(iconified_title_relief,
+						     normal_title_relief) +
+						 Ffont->height - 1,
+						 IconShadowGC, IconReliefGC,
+						 title_relief );
+			}
+
 		}
 		if (region)
 		{
@@ -1030,6 +1119,18 @@ void animate(struct icon_info *item, unsigned long *body)
     int abs_x, abs_y;
     Window junkw;
 
+    /* do not animate if shaded */
+    if (Swallowed)
+    {
+	    if (SwallowerShaded)
+	    {
+		    return;
+	    }
+    }
+    else if (Shaded)
+    {
+	    return;
+    }
     /* check to see if the centre of the icon is displayed in the iconbox */
     XTranslateCoordinates(dpy, icon_win, main_win, item->x + (item->icon_w/2),
 			  item->y + (item->icon_h/2), &abs_x, &abs_y, &junkw);
@@ -1055,11 +1156,11 @@ void animate(struct icon_info *item, unsigned long *body)
   }
 }
 
-/***********************************************************************
+/*
  * RedrawHScrollbar
  *      Based on part of Loop() of GrabWindow.c in FvwmScroll:
  *              Copyright 1994, Robert Nation.
- ***********************************************************************/
+ */
 void RedrawHScrollbar(void)
 {
   int x,width;
@@ -1071,11 +1172,11 @@ void RedrawHScrollbar(void)
 		   ReliefGC, ShadowGC, 2);
 }
 
-/***********************************************************************
+/*
  * RedrawVScrollbar
  *      Based on part of Loop() of GrabWindow.c in FvwmScroll:
  *              Copyright 1994, Robert Nation.
- ***********************************************************************/
+ */
 void RedrawVScrollbar(void)
 {
   int y, height;
@@ -1196,11 +1297,11 @@ void RedrawBottomButton(GC rgc, GC sgc)
   XDrawSegments(dpy, b_button, sgc, seg, i);
 }
 
-/************************************************************************
+/*
  * CreateWindow --Sizes and creates the window
  *      Based on CreateWindow() from GoodStuff:
  *              Copyright 1993, Robert Nation.
- ***********************************************************************/
+ */
 void CreateWindow(void)
 {
   XGCValues gcv;
@@ -1211,6 +1312,8 @@ void CreateWindow(void)
   XSizeHints mysizehints;
   XTextProperty name;
   XClassHint class_hints;
+  int wx = 0;
+  int wy = 0;
 
   h_margin = margin1*2 + bar_width + margin2 + 8;
   v_margin = margin1*2 + bar_width + margin2 + 8;
@@ -1231,8 +1334,11 @@ void CreateWindow(void)
   if ((local_flags & HIDE_V))
     h_margin -= bar_width + margin2 + 4;
 
+  /* calculate the (maximal) width and height of an icon */
   UWidth = max_icon_width + icon_relief + interval;
-  UHeight = Ffont->height + max_icon_height + icon_relief + 6 + interval;
+  UHeight = Ffont->height + 2 +
+	  2 * max(normal_title_relief,iconified_title_relief) +
+	  max_icon_height + icon_relief + interval;
   Width = UWidth * num_columns + interval -1;
   Height = UHeight * num_rows + interval -1;
 
@@ -1250,21 +1356,21 @@ void CreateWindow(void)
   {
     if (xneg)
     {
-      mysizehints.x = DisplayWidth(dpy,screen) + geom_x - mysizehints.width;
+      wx = DisplayWidth(dpy,screen) + geom_x - mysizehints.width;
       gravity = NorthEastGravity;
     }
     else
     {
-      mysizehints.x = geom_x;
+      wx = geom_x;
     }
     if (yneg)
     {
-      mysizehints.y = DisplayHeight(dpy,screen) + geom_y - mysizehints.height;
+      wy = DisplayHeight(dpy,screen) + geom_y - mysizehints.height;
       gravity = SouthWestGravity;
     }
     else
     {
-      mysizehints.y = geom_y;
+      wy = geom_y;
     }
 
     if (xneg && yneg)
@@ -1273,7 +1379,10 @@ void CreateWindow(void)
     }
 
     mysizehints.flags |= USPosition;
+    /* hack to prevent mapping on wrong screen with StartsOnScreen */
+    FScreenMangleScreenIntoUSPosHints(FSCREEN_XYPOS, &mysizehints);
   }
+/*!!!*/fprintf(stderr, "pos %d %d, grav %d\n", wx, wy, gravity);
 
   mysizehints.win_gravity = gravity;
 
@@ -1313,7 +1422,7 @@ void CreateWindow(void)
   attributes.background_pixel = back_pix;
   attributes.border_pixel = 0;
   attributes.colormap = Pcmap;
-  main_win = XCreateWindow(dpy, Root, mysizehints.x, mysizehints.y,
+  main_win = XCreateWindow(dpy, Root, wx, wy,
 			   mysizehints.width, mysizehints.height, 0, Pdepth,
 			   InputOutput, Pvisual,
 			   CWBackPixel | CWBorderPixel | CWColormap,
@@ -1585,11 +1694,6 @@ void GetIconwinSize(int *dx, int *dy)
   *dy = icon_win_height - *dy;
 }
 
-void nocolor(char *a, char *b)
-{
- fprintf(stderr,"%s: can't %s %s\n", MyName, a,b);
-}
-
 void MySendFvwmPipe(int *fd, char *message, unsigned long window)
 {
   long w;
@@ -1742,7 +1846,7 @@ void Prev(void)
   }
 }
 
-/************************************************************************
+/*
  * TerminateHandler - signal handler to make FvwmIconBox exit cleanly
  */
 static RETSIGTYPE
@@ -1754,19 +1858,21 @@ TerminateHandler(int sig)
    * BEFORE we call it ...
    */
   fvwmSetTerminate(sig);
+  SIGNAL_RETURN;
 }
 
 
-/************************************************************************
+/*
  * DeadPipe --Dead pipe handler
  *      Based on DeadPipe() from GoodStuff:
  *              Copyright 1993, Robert Nation.
- ***********************************************************************/
+ */
 void
 DeadPipe(int nonsense)
 {
   is_dead_pipe = True;
   exit(0);
+  SIGNAL_RETURN;
 }
 
 static void
@@ -1845,11 +1951,11 @@ CleanUp(void)
 }
 
 
-/************************************************************************
+/*
  * ParseOptions
  *      Based on ParseConfig() from FvwmWinList:
  *              Copyright 1994, Mike Finger.
- ***********************************************************************/
+ */
 void ParseOptions(void)
 {
   char *tline= NULL,*tmp;
@@ -1892,6 +1998,7 @@ void ParseOptions(void)
 	  xneg = 1;
 	if (flags & YNegative)
 	  yneg = 1;
+/*!!!*/fprintf(stderr, "geom: size %d %d, pos %d %d, neg %d %d\n", num_columns, num_rows, geom_x, geom_y, xneg, yneg);
       }
       else if (strncasecmp(tline,CatString3(
 			     "*", MyName, "MaxIconSize"),Clength+12)==0)
@@ -1912,6 +2019,41 @@ void ParseOptions(void)
 	  max_icon_width += 4;
 	}
       }
+      else if (strncasecmp(tline, CatString3(
+			   "*", MyName, "NoIconifiedParentheses"),
+			   Clength+23) == 0)
+      {
+	iconified_has_normal_title = True;
+      }
+
+      else if (strncasecmp(tline, CatString3(
+			   "*", MyName, "NormalTitleRelief"),
+			   Clength+18) == 0)
+      {
+	normal_title_relief = atoi(&tline[Clength+18]);
+      }
+
+      else if (strncasecmp(tline, CatString3(
+			   "*", MyName, "IconifiedTitleRelief"),
+			   Clength+21) == 0)
+      {
+	iconified_title_relief = atoi(&tline[Clength+21]);
+      }
+
+      else if (strncasecmp(tline, CatString3(
+			   "*", MyName, "NormalTitleInvertedRelief"),
+			   Clength+26) == 0)
+      {
+	normal_title_inverted_relief = True;
+      }
+
+      else if (strncasecmp(tline, CatString3(
+			   "*", MyName, "IconifiedTitleInvertedRelief"),
+			   Clength+29) == 0)
+      {
+	iconified_title_inverted_relief = True;
+      }
+
       else if (strncasecmp(tline, "Colorset", 8) == 0)
       {
 	LoadColorset(&tline[8]);
@@ -2098,10 +2240,6 @@ void ParseOptions(void)
       {
 	ClickTime = atoi(&tline[9]);
       }
-      else if (strncasecmp(tline,"ColorLimit",10)==0)
-      {
-	save_color_limit = atoi(&tline[10]);
-      }
     }
     GetConfigLine(fd,&tline);
   }
@@ -2209,7 +2347,7 @@ void parsemouse(char *tline)
   while((!isspace((unsigned char)*end))&&(*end!='\n')&&(*end!=0))
     end++;
   n = sscanf(start, "%d", &b);
-  if (n > 0 && b >= 0 && b <= NUMBER_OF_MOUSE_BUTTONS)
+  if (n > 0 && b >= 0 && b <= NUMBER_OF_EXTENDED_MOUSE_BUTTONS)
     f->mouse = b;
   else
     f->mouse = -1;
@@ -2253,13 +2391,13 @@ void parsemouse(char *tline)
   MouseActions = f;
 }
 
-/***********************************************************************
+/*
   parsekey
 	Based on part of AddFunckey() of configure.c in Fvwm.
 	Copyright 1988, Evans and Sutherland Computer Corporation,
 	Copyright 1989, Massachusetts Institute of Technology,
 	Copyright 1993, Robert Nation.
- ***********************************************************************/
+*/
 void parsekey(char *tline)
 {
   struct keyfunc *k;
@@ -2320,11 +2458,11 @@ void parsekey(char *tline)
       }
 }
 
-/***********************************************************************
+/*
  * change_window_name
  *      Original work from GoodStuff:
  *              Copyright 1993, Robert Nation.
- ***********************************************************************/
+ */
 void change_window_name(char *str)
 {
   XTextProperty name;
@@ -2339,11 +2477,11 @@ void change_window_name(char *str)
   XFree(name.value);
 }
 
-/***********************************************************************
+/*
  * My_XNextEvent
  *      Original work from GoodStuff:
  *              Copyright 1993, Robert Nation.
- ***********************************************************************/
+ */
 int My_XNextEvent(Display *dpy, XEvent *event)
 {
   fd_set in_fdset;
@@ -2388,11 +2526,11 @@ int My_XNextEvent(Display *dpy, XEvent *event)
   return 0;
 }
 
-/**************************************************************************
+/*
  * process_message
  *      Based on ProcessMassage() from FvwmWinList:
  *              Copyright 1994, Mike Finger.
- *************************************************************************/
+ */
 int diffx, diffy;
 void process_message(unsigned long type, unsigned long *body)
 {
@@ -2404,6 +2542,14 @@ void process_message(unsigned long type, unsigned long *body)
   switch(type)
   {
   case M_CONFIGURE_WINDOW:
+    if (cfgpacket->w == main_win)
+    {
+	    Shaded = IS_SHADED(cfgpacket);
+    }
+    if (SwallowerWin && cfgpacket->w == SwallowerWin)
+    {
+	    SwallowerShaded = IS_SHADED(cfgpacket);
+    }
     if (ready)
     {
       tmp = Head;
@@ -2431,7 +2577,7 @@ void process_message(unsigned long type, unsigned long *body)
 	    SET_DO_SKIP_WINDOW_LIST(tmp,DO_SKIP_WINDOW_LIST(cfgpacket));
 	  }
 	  if ((local_flags & CURRENT_ONLY) &&
-	      tmp->desk != cfgpacket->desk && !IS_STICKY_ON_DESK(tmp))
+	      tmp->desk != cfgpacket->desk && !IS_STICKY_ACROSS_DESKS(tmp))
 	  {
 	    olddesk = tmp->desk;
 	    tmp->desk = cfgpacket->desk;
@@ -2444,12 +2590,12 @@ void process_message(unsigned long type, unsigned long *body)
 	      else
 		remove = 1;
 	    }
-	  }else if ((IS_STICKY_ON_DESK(cfgpacket)) &&
-		    !(IS_STICKY_ON_DESK(tmp))) /* stick */
-	    SET_STICKY_ON_DESK(tmp, True);
-	  else if (!(IS_STICKY_ON_DESK(cfgpacket)) &&
-		   (IS_STICKY_ON_DESK(tmp))){ /* unstick */
-	    SET_STICKY_ON_DESK(tmp, False);
+	  }else if ((IS_STICKY_ACROSS_DESKS(cfgpacket)) &&
+		    !(IS_STICKY_ACROSS_DESKS(tmp))) /* stick */
+	    SET_STICKY_ACROSS_DESKS(tmp, True);
+	  else if (!(IS_STICKY_ACROSS_DESKS(cfgpacket)) &&
+		   (IS_STICKY_ACROSS_DESKS(tmp))){ /* unstick */
+	    SET_STICKY_ACROSS_DESKS(tmp, False);
 	    tmp->desk = cfgpacket->desk;
 	  }
 	  if ((add && window_cond(tmp)) || remove) {
@@ -2688,7 +2834,23 @@ void process_message(unsigned long type, unsigned long *body)
     }
     else if  (body[0] == MX_PROPERTY_CHANGE_SWALLOW && body[2] == icon_win)
     {
-      Swallowed = body[1];
+	    char *str;
+
+	    Swallowed = body[1];
+	    str = (char *)&body[3];
+
+	    if (Swallowed && str)
+	    {
+		    unsigned long u;
+		    if (sscanf(str,"%lu",&u) == 1)
+		    {
+			    SwallowerWin = (Window)u;
+		    }
+		    else
+		    {
+			    SwallowerWin = 0;
+		    }
+	    }
     }
     break;
   default:
@@ -2746,7 +2908,7 @@ int AdjustIconWindows(void)
 int window_cond(struct icon_info *item)
 {
   if ((!(local_flags & CURRENT_ONLY) ||
-      (IS_STICKY_ON_DESK(item)) || (item->desk == CurrentDesk)) &&
+      (IS_STICKY_ACROSS_DESKS(item)) || (item->desk == CurrentDesk)) &&
       IS_ICON_SUPPRESSED(item) &&
       (!DO_SKIP_WINDOW_LIST(item) || !UseSkipList))
     return 1;
@@ -2754,11 +2916,11 @@ int window_cond(struct icon_info *item)
   return 0;
 }
 
-/************************************************************************
+/*
  * AddItem
  *      Skeleton based on AddItem() from FvwmWinList:
  *              Copyright 1994, Mike Finger.
- ***********************************************************************/
+ */
 /*Bool AddItem(unsigned long id, long desk, unsigned long flags) */
 Bool AddItem(ConfigWinPacket *cfgpacket)
 {
@@ -2811,11 +2973,11 @@ Bool AddItem(ConfigWinPacket *cfgpacket)
   return False;
 }
 
-/************************************************************************
+/*
  * Deletetem
  *      Skeleton based on DeleteItem() from FvwmWinList:
  *              Copyright 1994, Mike Finger.
- ***********************************************************************/
+ */
 Bool DeleteItem(unsigned long id)
 {
   struct icon_info *tmp = Head;
@@ -2851,11 +3013,11 @@ Bool DeleteItem(unsigned long id)
   return False;
 }
 
-/************************************************************************
+/*
  * UpdateItem
  *      Skeleton based on UpdateItem() from FvwmWinList:
  *              Copyright 1994, Mike Finger.
- ***********************************************************************/
+ */
 struct icon_info *UpdateItem(unsigned long type, unsigned long id, char *item)
 {
   struct icon_info *tmp;
@@ -3132,10 +3294,10 @@ void freeitem(struct icon_info *item, int d)
 }
 
 
-/************************************************************************
+/*
  * CheckActionType
  *   Based on functions.c from Fvwm:
- ***********************************************************************/
+ */
 static int CheckActionType(
   int x, int y, XEvent *d, Bool may_time_out, Bool is_button_pressed)
 {
@@ -3200,13 +3362,13 @@ static int CheckActionType(
 }
 
 
-/************************************************************************
+/*
  * ExecuteAction
 *       Based on part of ComplexFunction() of functions.c from fvwm:
 	Copyright 1988, Evans and Sutherland Computer Corporation,
 	Copyright 1989, Massachusetts Institute of Technology,
 	Copyright 1993, Robert Nation.
- ***********************************************************************/
+*/
 void ExecuteAction(int x, int y, struct icon_info *item)
 {
   int type = NO_CLICK;
@@ -3273,13 +3435,13 @@ void ExecuteKey(XEvent event)
   }
 }
 
-/***********************************************************************
+/*
  LookInList
 	Based on part of LookInList() of add_window.c from fvwm:
 	Copyright 1988, Evans and Sutherland Computer Corporation,
 	Copyright 1989, Massachusetts Institute of Technology,
 	Copyright 1993, Robert Nation.
- ***********************************************************************/
+*/
 int LookInList(struct icon_info *item)
 {
   int isdefault=1;
@@ -3354,13 +3516,13 @@ int LookInList(struct icon_info *item)
   return 1;
 }
 
-/***********************************************************************
+/*
  strcpy
 	Based on stripcpy2() of configure.c from Fvwm:
 	Copyright 1988, Evans and Sutherland Computer Corporation,
 	Copyright 1989, Massachusetts Institute of Technology,
 	Copyright 1993, Robert Nation.
- ***********************************************************************/
+*/
 char *stripcpy2(char *source)
 {
   char *ptr;
@@ -3382,9 +3544,9 @@ char *stripcpy2(char *source)
   return ptr;
 }
 
-/***********************************************************************
+/*
  Error handler
- ***********************************************************************/
+*/
 static int
 myErrorHandler(Display *dpy, XErrorEvent *event)
 {
