@@ -137,6 +137,47 @@ Pixmap default_pixmap = None;
 #endif
 
 
+#define  MAX_UNPROCESSED_MESSAGES 1
+/* sums up pixels to scroll. If do_send_message is True a Scroll command is
+ * sent back to fvwm. The function shall be called with is_message_recieved
+ * True when the Scroll command has been processed by fvwm. This is checked
+ * by talking to ourself. */
+static void do_scroll(int sx, int sy, Bool do_send_message,
+		      Bool is_message_recieved)
+{
+	static int psx = 0;
+	static int psy = 0;
+	static int messages_sent = 0;
+	char command[256];
+	psx+=sx;
+	psy+=sy;
+	if (is_message_recieved)
+	{
+		/* There might be other modules with the same name, or someone
+		 might send ScrollDone other than the module, so just treat
+		 any negative count as zero. */
+		if (--messages_sent < 0)
+		{
+			messages_sent = 0;
+		}
+	}
+	if ((do_send_message || messages_sent < MAX_UNPROCESSED_MESSAGES) &&
+	    ( psx != 0 || psy != 0 ))
+	{
+		sprintf(command, "Scroll %dp %dp", psx, psy);
+		SendText(fd, command, 0);
+		messages_sent++;
+		SendText(fd, "Send_Reply ScrollDone", 0);
+		psx = 0;
+		psy = 0;
+	}
+}
+
+void HandleScrollDone()
+{
+	do_scroll(0, 0, True, True);
+}
+
 /* discard certain events on a window */
 static void discard_events(long event_type, Window w, XEvent *last_ev)
 {
@@ -161,8 +202,6 @@ static void discard_events(long event_type, Window w, XEvent *last_ev)
  *	CalcGeom - calculates the size and position of a mini-window
  *	given the real window size.
  *	You can always tell bad code by the size of the comments.
- *
- *	dv: Some people say that good code does not nee lengthy comments. :-)
  */
 static void CalcGeom(PagerWindow *t, int win_w, int win_h,
 		     int *x_ret, int *y_ret, int *w_ret, int *h_ret)
@@ -179,11 +218,11 @@ static void CalcGeom(PagerWindow *t, int win_w, int win_h,
   virt += t->width - 1;
 
   /* to calculate the right edge, mirror the window and use the same
-   * calculations as for the loeft edge for consistency. */
+   * calculations as for the left edge for consistency. */
   virt2 = Scr.VWidth - 1 - virt;
   edge2 = (virt2 * win_w) / Scr.VWidth;
 
-  /* then mirror if back to get the real coordinate */
+  /* then mirror it back to get the real coordinate */
   edge2 = win_w - 1 - edge2;
 
   /* Calculate the mini-window's width by subtracting its LHS
@@ -917,7 +956,7 @@ void initialize_pager(void)
 }
 
 
-void UpdateWindowShape ()
+void UpdateWindowShape(void)
 {
   if (FHaveShapeExtension)
   {
@@ -1076,6 +1115,8 @@ void DispatchEvent(XEvent *Event)
 	}
 	Scroll(icon_w, icon_h, x, y, 0, True);
       }
+      /* Flush any pending scroll operations */
+      do_scroll(0, 0, True, False);
     }
     else if((Event->xbutton.button == 1)||
 	    (Event->xbutton.button == 2))
@@ -1606,6 +1647,8 @@ void ReConfigureIcons(Bool do_reconfigure_desk_only)
     if (do_reconfigure_desk_only && t->desk != Scr.CurrentDesk)
       continue;
     CalcGeom(t, icon_w, icon_h, &x, &y, &w, &h);
+    t->icon_view_x = x;
+    t->icon_view_y = y;
     t->icon_view_width = w;
     t->icon_view_height = h;
     if(Scr.CurrentDesk == t->desk)
@@ -1961,6 +2004,8 @@ void AddNewWindow(PagerWindow *t)
 	}
 
 	CalcGeom(t, icon_w, icon_h, &x, &y, &w, &h);
+	t->icon_view_x = x;
+	t->icon_view_y = y;
 	t->icon_view_width = w;
 	t->icon_view_height = h;
 	if(Scr.CurrentDesk != t->desk)
@@ -1999,7 +2044,7 @@ void AddNewWindow(PagerWindow *t)
 	{
 		t->myflags.is_mapped = 0;
 	}
-	Hilight(t,False);
+	Hilight(t, False);
 
 	return;
 }
@@ -2050,6 +2095,8 @@ void ChangeDeskForWindow(PagerWindow *t,long newdesk)
 
   CalcGeom(t, icon_w, icon_h, &x, &y, &w, &h);
   size_changed = (t->icon_view_width != w || t->icon_view_height != h);
+  t->icon_view_x = x;
+  t->icon_view_y = y;
   t->icon_view_width = w;
   t->icon_view_height = h;
   if(Scr.CurrentDesk != t->desk)
@@ -2123,10 +2170,10 @@ void MoveResizePagerView(PagerWindow *t, Bool do_force_redraw)
   }
 
   CalcGeom(t, icon_w, icon_h, &x, &y, &w, &h);
-  position_changed = (t->pager_view_x != x || t->pager_view_y != y);
+  position_changed = (t->icon_view_x != x || t->icon_view_y != y);
   size_changed = (t->icon_view_width != w || t->icon_view_height != h);
-  t->pager_view_x = x;
-  t->pager_view_y = y;
+  t->icon_view_x = x;
+  t->icon_view_y = y;
   t->icon_view_width = w;
   t->icon_view_height = h;
   if (Scr.CurrentDesk == t->desk)
@@ -2245,7 +2292,6 @@ void Hilight(PagerWindow *t, int on)
 void Scroll(int window_w, int window_h, int x, int y, int Desk,
 	    Bool do_scroll_icon)
 {
-	char command[256];
 	static int last_sx = -999999;
 	static int last_sy = -999999;
 	int sx;
@@ -2261,8 +2307,8 @@ void Scroll(int window_w, int window_h, int x, int y, int Desk,
 	/* center around mouse */
 	adjx = (desk_w / (1 + Scr.VxMax / Scr.MyDisplayWidth));
 	adjy = (desk_h / (1 + Scr.VyMax / Scr.MyDisplayHeight));
-	x-=adjx/2;
-	y-=adjy/2;
+	x -= adjx/2;
+	y -= adjy/2;
 
 	/* adjust for pointer going out of range */
 	if (x < 0)
@@ -2287,44 +2333,32 @@ void Scroll(int window_w, int window_h, int x, int y, int Desk,
 	sy = 0;
 	if (window_w != 0)
 	{
-		sx = 100 * (x * Scr.VWidth / window_w - MyVx) /
-			Scr.MyDisplayWidth;
+		sx = (x * Scr.VWidth / window_w - MyVx);
 	}
 	if (window_h != 0)
 	{
-		sy = 100 * (y * Scr.VHeight / window_h - MyVy) /
-			Scr.MyDisplayHeight;
-	}
-	MYFPRINTF((stderr,"[scroll]: %d %d %d %d %d %d\n", window_w, window_h, x,
-		y, sx, sy));
-	/* Make sure weExecuteCommandQueue(); don't get stuck a few pixels from
-	 * the top/left border. Since sx/sy are ints, values between 0 and 1 are
-	 * rounded down. */
-	if (sx == 0 && x == 0 && MyVx != 0)
-	{
-		sx = -1;
-	}
-	if (sy == 0 && y == 0 && MyVy != 0)
-	{
-		sy = -1;
+		sy = (y * Scr.VHeight / window_h - MyVy);
 	}
+	MYFPRINTF((stderr,"[scroll]: %d %d %d %d %d %d\n", window_w, window_h,
+		   x, y, sx, sy));
 	if (sx == 0 && sy == 0)
 	{
 		return;
 	}
 	if (Wait == 0 || last_sx != sx || last_sy != sy)
 	{
-		sprintf(command, "Scroll %d %d", sx, sy);
-		SendText(fd, command, 0);
+		do_scroll(sx, sy, False, False);
+
 		/* Here we need to track the view offset on the desk. */
-		/* sx/y are are a percent of the screen to scroll. */
+		/* sx/y are are pixels on the screen to scroll. */
 		/* We don't use Scr.Vx/y since they lag the true position. */
-		MyVx += Scr.MyDisplayWidth * sx / 100;
+		MyVx += sx;
 		if (MyVx < 0)
 		{
 			MyVx = 0;
 		}
-		MyVy += Scr.MyDisplayHeight * sy / 100;
+		MyVy += sy;
+
 		if (MyVy < 0)
 		{
 			MyVy = 0;
@@ -2342,270 +2376,306 @@ void Scroll(int window_w, int window_h, int x, int y, int Desk,
 
 void MoveWindow(XEvent *Event)
 {
-  char command[100];
-  int x1,y1,finished = 0,wx,wy,n,x,y,xi=0,yi=0,wx1,wy1,x2,y2;
-  Window dumwin;
-  PagerWindow *t;
-  int m,n1,m1;
-  int NewDesk,KeepMoving = 0;
-  int moved = 0;
-  int row,column;
-  Window JunkRoot, JunkChild;
-  int JunkX, JunkY;
-  unsigned JunkMask;
-  int do_switch_desk_later = 0;
+	char command[100];
+	int x1, y1, wx, wy, n, x, y, xi = 0, yi = 0, wx1, wy1, x2, y2;
+	int finished = 0;
+	Window dumwin;
+	PagerWindow *t;
+	int m, n1, m1;
+	int NewDesk, KeepMoving = 0;
+	int moved = 0;
+	int row, column;
+	Window JunkRoot, JunkChild;
+	int JunkX, JunkY;
+	unsigned JunkMask;
+	int do_switch_desk_later = 0;
 
-  t = Start;
-  while ((t != NULL)&&(t->PagerView != Event->xbutton.subwindow))
-    t= t->next;
-
-  if(t==NULL)
-  {
-    t = Start;
-    while ((t != NULL)&&(t->IconView != Event->xbutton.subwindow))
-      t= t->next;
-    if(t!=NULL)
-    {
-      IconMoveWindow(Event,t);
-      return;
-    }
-  }
-
-  if(t == NULL)
-    return;
-
-  NewDesk = t->desk - desk1;
-  if((NewDesk < 0)||(NewDesk >= ndesks))
-    return;
-
-  n = Scr.VxMax / Scr.MyDisplayWidth;
-  m = Scr.VyMax / Scr.MyDisplayHeight;
-  n1 = (Scr.Vx + t->x) / Scr.MyDisplayWidth;
-  m1 = (Scr.Vy + t->y) / Scr.MyDisplayHeight;
-  wx = (Scr.Vx + t->x) * (desk_w - n) / Scr.VWidth + n1;
-  wy = (Scr.Vy + t->y) * (desk_h - m) / Scr.VHeight + m1;
-  wx1 = wx + (desk_w + 1) * (NewDesk % Columns);
-  wy1 = wy + label_h + (desk_h + label_h + 1) * (NewDesk / Columns);
-  if (LabelsBelow)
-  {
-    wy1 -= label_h;
-  }
-
-  XReparentWindow(dpy, t->PagerView, Scr.Pager_w, wx1, wy1);
-  XRaiseWindow(dpy,t->PagerView);
-
-  XTranslateCoordinates(dpy, Event->xany.window, t->PagerView,
-			Event->xbutton.x, Event->xbutton.y, &x1, &y1, &dumwin);
-  xi = x1;
-  yi = y1;
-  while(!finished)
-  {
-    FMaskEvent(dpy,ButtonReleaseMask | ButtonMotionMask|ExposureMask,Event);
-
-    if(Event->type == MotionNotify)
-    {
-      XTranslateCoordinates(dpy, Event->xany.window, Scr.Pager_w,
-			    Event->xmotion.x, Event->xmotion.y, &x, &y,
-			    &dumwin);
-      if(moved == 0)
-      {
-	xi = x;
-	yi = y;
-	moved = 1;
-      }
-      if((x < -5)||(y<-5)||(x>window_w+5)||(y>window_h+5))
-      {
-	KeepMoving = 1;
-	finished = 1;
-      }
-      XMoveWindow(dpy, t->PagerView, x - (x1), y - (y1));
-    }
-    else if(Event->type == ButtonRelease)
-    {
-      XTranslateCoordinates(dpy, Event->xany.window, Scr.Pager_w,
-			    Event->xbutton.x, Event->xbutton.y, &x, &y,
-			    &dumwin);
-      XMoveWindow(dpy, t->PagerView, x - x1, y - y1);
-      finished = 1;
-    }
-    else if (Event->type == Expose)
-    {
-      HandleExpose(Event);
-    }
-  }
-
-  if(moved)
-  {
-    if((x - xi < MoveThreshold)&&(y - yi < MoveThreshold)&&
-       (x - xi > -MoveThreshold)&&(y -yi > -MoveThreshold))
-      moved = 0;
-  }
-  if(KeepMoving)
-  {
-    NewDesk = Scr.CurrentDesk;
-    if(NewDesk != t->desk)
-    {
-      XMoveWindow(dpy, IS_ICONIFIED(t) ? t->icon_w : t->w,
-		  Scr.VWidth, Scr.VHeight);
-      XSync(dpy, False);
-    }
-    if((NewDesk>=desk1)&&(NewDesk<=desk2))
-      XReparentWindow(dpy, t->PagerView, Desks[NewDesk-desk1].w,0,0);
-    else
-    {
-      XDestroyWindow(dpy,t->PagerView);
-      t->PagerView = None;
-    }
-    if (FQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
-		      &x, &y, &JunkX, &JunkY, &JunkMask) == False)
-    {
-      /* pointer is on a different screen */
-      x = 0;
-      y = 0;
-    }
-    XUngrabPointer(dpy,CurrentTime);
-    XSync(dpy,0);
-    sprintf(command, "Silent Move %dp %dp", x, y);
-    SendText(fd, command, t->w);
-    if (NewDesk != t->desk)
-    {
-      sprintf(command, "Silent MoveToDesk 0 %d", NewDesk);
-      SendText(fd, command, t->w);
-      t->desk = NewDesk;
-    }
-    SendText(fd, "Silent Raise", t->w);
-    SendText(fd, "Silent Move Pointer", t->w);
-    return;
-  }
-  else
-  {
-    column = (x/(desk_w+1));
-    if (column >= Columns)
-      column = Columns - 1;
-    if (column < 0)
-      column = 0;
-    row =  (y/(desk_h+ label_h+1));
-    if (row >= Rows)
-      row = Rows - 1;
-    if (row < 0)
-      row = 0;
-    NewDesk = column + (row)*Columns;
-    while (NewDesk < 0)
-    {
-      NewDesk += Columns;
-      if (NewDesk >= ndesks)
-	NewDesk = 0;
-    }
-    while (NewDesk >= ndesks)
-    {
-      NewDesk -= Columns;
-      if (NewDesk < 0)
-	NewDesk = ndesks - 1;
-    }
-    XTranslateCoordinates(dpy, Scr.Pager_w,Desks[NewDesk].w,
-			  x-x1, y-y1, &x2,&y2,&dumwin);
-
-    n1 = x2 * Scr.VWidth / (desk_w * Scr.MyDisplayWidth);
-    m1 = y2* Scr.VHeight / (desk_h * Scr.MyDisplayHeight);
-    x = (x2 - n1) * Scr.VWidth / (desk_w - n) - Scr.Vx;
-    y = (y2 - m1) * Scr.VHeight / (desk_h - m) - Scr.Vy;
-    /* force onto desk */
-    if (x + t->frame_width + Scr.Vx < 0 )
-      x = - Scr.Vx - t->frame_width;
-    if (y + t->frame_height + Scr.Vy < 0)
-      y = - Scr.Vy - t->frame_height;
-    if (x + Scr.Vx >= Scr.VWidth)
-      x = Scr.VWidth - Scr.Vx - 1;
-    if (y + Scr.Vy >= Scr.VHeight)
-      y = Scr.VHeight - Scr.Vy - 1;
-    if((IS_ICONIFIED(t) && IS_ICON_STICKY_ACROSS_DESKS(t)) ||
-       (IS_STICKY_ACROSS_DESKS(t)))
-    {
-      NewDesk = Scr.CurrentDesk - desk1;
-      if (x > Scr.MyDisplayWidth - 16)
-	x = Scr.MyDisplayWidth - 16;
-      if (y > Scr.MyDisplayHeight - 16)
-	y = Scr.MyDisplayHeight - 16;
-      if (x + t->width < 16)
-	x = 16 - t->width;
-      if (y + t->height < 16)
-	y = 16 - t->height;
-    }
-    if(NewDesk + desk1 != t->desk)
-    {
-      if((IS_ICONIFIED(t) && IS_ICON_STICKY_ACROSS_DESKS(t)) ||
-	 (IS_STICKY_ACROSS_DESKS(t)))
-      {
-	NewDesk = Scr.CurrentDesk - desk1;
-	if(t->desk != Scr.CurrentDesk)
-	  ChangeDeskForWindow(t,Scr.CurrentDesk);
-      }
-      else if (NewDesk + desk1 != Scr.CurrentDesk)
-      {
-	sprintf(command, "Silent MoveToDesk 0 %d", NewDesk + desk1);
-	SendText(fd, command, t->w);
-	t->desk = NewDesk + desk1;
-      }
-      else
-      {
-	do_switch_desk_later = 1;
-      }
-    }
-
-    if((NewDesk >= 0)&&(NewDesk < ndesks))
-    {
-      XReparentWindow(dpy, t->PagerView, Desks[NewDesk].w,x2,y2);
-      t->desk = NewDesk;
-      XClearArea(dpy, t->PagerView, 0, 0, 0, 0, True);
-      if(moved)
-      {
-	if(IS_ICONIFIED(t))
+	t = Start;
+	while (t != NULL && t->PagerView != Event->xbutton.subwindow)
 	{
-	  XMoveWindow(dpy,t->icon_w,x,y);
+		t = t->next;
+	}
+
+	if (t == NULL)
+	{
+		t = Start;
+		while (t != NULL && t->IconView != Event->xbutton.subwindow)
+		{
+			t = t->next;
+		}
+		if (t != NULL)
+		{
+			IconMoveWindow(Event, t);
+			return;
+		}
+	}
+
+	if (t == NULL || !t->allowed_actions.is_movable)
+	{
+		return;
+	}
+
+	NewDesk = t->desk - desk1;
+	if (NewDesk < 0 || NewDesk >= ndesks)
+	{
+		return;
+	}
+
+	n = Scr.VxMax / Scr.MyDisplayWidth;
+	m = Scr.VyMax / Scr.MyDisplayHeight;
+	n1 = (Scr.Vx + t->x) / Scr.MyDisplayWidth;
+	m1 = (Scr.Vy + t->y) / Scr.MyDisplayHeight;
+	wx = (Scr.Vx + t->x) * (desk_w - n) / Scr.VWidth + n1;
+	wy = (Scr.Vy + t->y) * (desk_h - m) / Scr.VHeight + m1;
+	wx1 = wx + (desk_w + 1) * (NewDesk % Columns);
+	wy1 = wy + label_h + (desk_h + label_h + 1) * (NewDesk / Columns);
+	if (LabelsBelow)
+	{
+		wy1 -= label_h;
+	}
+
+	XReparentWindow(dpy, t->PagerView, Scr.Pager_w, wx1, wy1);
+	XRaiseWindow(dpy, t->PagerView);
+
+	XTranslateCoordinates(dpy, Event->xany.window, t->PagerView,
+			      Event->xbutton.x, Event->xbutton.y, &x1, &y1,
+			      &dumwin);
+	xi = x1;
+	yi = y1;
+	while (!finished)
+	{
+		FMaskEvent(dpy, ButtonReleaseMask | ButtonMotionMask |
+			   ExposureMask, Event);
+		if (Event->type == MotionNotify)
+		{
+			XTranslateCoordinates(dpy, Event->xany.window,
+					      Scr.Pager_w, Event->xmotion.x,
+					      Event->xmotion.y, &x, &y,
+					      &dumwin);
+
+			if (moved == 0)
+			{
+				xi = x;
+				yi = y;
+				moved = 1;
+			}
+			if (x < -5 || y < -5 || x > window_w + 5 ||
+			   y > window_h + 5)
+			{
+				KeepMoving = 1;
+				finished = 1;
+			}
+			XMoveWindow(dpy, t->PagerView, x - x1, y - y1);
+		}
+		else if (Event->type == ButtonRelease)
+		{
+			XTranslateCoordinates(dpy, Event->xany.window,
+					      Scr.Pager_w, Event->xbutton.x,
+					      Event->xbutton.y, &x, &y,
+					      &dumwin);
+			XMoveWindow(dpy, t->PagerView, x - x1, y - y1);
+			finished = 1;
+		}
+		else if (Event->type == Expose)
+		{
+			HandleExpose(Event);
+		}
+	}
+
+	if (moved)
+	{
+		if (abs(x - xi) < MoveThreshold &&
+		    abs(y - yi) < MoveThreshold)
+		{
+			moved = 0;
+		}
+	}
+	if (KeepMoving)
+	{
+		NewDesk = Scr.CurrentDesk;
+
+		if (NewDesk >= desk1 && NewDesk <= desk2)
+		{
+			XReparentWindow(dpy, t->PagerView,
+					Desks[NewDesk-desk1].w, 0, 0);
+		}
+		else
+		{
+			XDestroyWindow(dpy, t->PagerView);
+			t->PagerView = None;
+		}
+		if (FQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
+				  &x, &y, &JunkX, &JunkY, &JunkMask) == False)
+		{
+			/* pointer is on a different screen */
+			x = 0;
+			y = 0;
+		}
+		XUngrabPointer(dpy,CurrentTime);
+		XSync(dpy,0);
+
+		if (NewDesk != t->desk)
+		{
+			/* griph: This used to move to NewDesk, but NewDesk
+			 * is current desk, and if fvwm is on another
+			 * desk (due to async operation) we have to move
+			 * the window to it anyway or "Move Pointer" will
+			 * move an invisible window. */
+
+			SendText(fd, "Silent MoveToDesk", t->w);
+			t->desk = NewDesk;
+		}
+		SendText(fd, "Silent Raise", t->w);
+		SendText(fd, "Silent Move Pointer", t->w);
+		return;
 	}
 	else
 	{
-	  char buf[64];
-	  int tx = x + t->border_width;
-	  int ty = y + t->title_height + t->border_width;
+		column = x / (desk_w + 1);
+		if (column >= Columns)
+		{
+			column = Columns - 1;
+		}
+		if (column < 0)
+		{
+			column = 0;
+		}
+		row = y / (desk_h + label_h + 1);
+		if (row >= Rows)
+		{
+			row = Rows - 1;
+		}
+		if (row < 0)
+		{
+			row = 0;
+		}
+		NewDesk = column + row * Columns;
+		while (NewDesk < 0)
+		{
+			NewDesk += Columns;
+			if (NewDesk >= ndesks)
+			{
+				NewDesk = 0;
+			}
+		}
+		while (NewDesk >= ndesks)
+		{
+			NewDesk -= Columns;
+			if (NewDesk < 0)
+			{
+				NewDesk = ndesks - 1;
+			}
+		}
+		XTranslateCoordinates(dpy, Scr.Pager_w, Desks[NewDesk].w,
+				      x - x1, y - y1, &x2, &y2, &dumwin);
 
-	  if (tx < 0)
-	  {
-	    tx = tx + t->width - Scr.MyDisplayWidth;
-	  }
-	  if (ty < 0)
-	  {
-	    ty = ty + t->height - Scr.MyDisplayHeight;
-	  }
-	  sprintf(buf, "Silent Move %dp %dp", tx, ty);
-	  SendText(fd, buf, t->w);
+		n1 = x2 * Scr.VWidth / (desk_w * Scr.MyDisplayWidth);
+		m1 = y2 * Scr.VHeight / (desk_h * Scr.MyDisplayHeight);
+		x = (x2 - n1) * Scr.VWidth / (desk_w - n) - Scr.Vx;
+		y = (y2 - m1) * Scr.VHeight / (desk_h - m) - Scr.Vy;
+		/* force onto desk */
+		if (x + t->frame_width + Scr.Vx < 0 )
+		{
+			x = -Scr.Vx - t->frame_width;
+		}
+		if (y + t->frame_height + Scr.Vy < 0)
+		{
+			y = -Scr.Vy - t->frame_height;
+		}
+		if (x + Scr.Vx >= Scr.VWidth)
+		{
+			x = Scr.VWidth - Scr.Vx - 1;
+		}
+		if (y + Scr.Vy >= Scr.VHeight)
+		{
+			y = Scr.VHeight - Scr.Vy - 1;
+		}
+		if ((IS_ICONIFIED(t) && IS_ICON_STICKY_ACROSS_DESKS(t)) ||
+		   (IS_STICKY_ACROSS_DESKS(t)))
+		{
+			NewDesk = Scr.CurrentDesk - desk1;
+			if (x > Scr.MyDisplayWidth - 16)
+			{
+				x = Scr.MyDisplayWidth - 16;
+			}
+			if (y > Scr.MyDisplayHeight - 16)
+			{
+				y = Scr.MyDisplayHeight - 16;
+			}
+			if (x + t->width < 16)
+			{
+				x = 16 - t->width;
+			}
+			if (y + t->height < 16)
+			{
+				y = 16 - t->height;
+			}
+		}
+		if (NewDesk + desk1 != t->desk)
+		{
+			if ((IS_ICONIFIED(t) && IS_ICON_STICKY_ACROSS_DESKS(t))
+			   || (IS_STICKY_ACROSS_DESKS(t)))
+			{
+				NewDesk = Scr.CurrentDesk - desk1;
+				if (t->desk != Scr.CurrentDesk)
+				{
+					ChangeDeskForWindow(t,Scr.CurrentDesk);
+				}
+			}
+			else if (NewDesk + desk1 != Scr.CurrentDesk)
+			{
+				sprintf(command, "Silent MoveToDesk 0 %d",
+					NewDesk + desk1);
+				SendText(fd, command, t->w);
+				t->desk = NewDesk + desk1;
+			}
+			else
+			{
+				do_switch_desk_later = 1;
+			}
+		}
+
+		if (NewDesk >= 0 && NewDesk < ndesks)
+		{
+			XReparentWindow(dpy, t->PagerView,
+					Desks[NewDesk].w, x2, y2);
+			t->desk = NewDesk;
+			XClearArea(dpy, t->PagerView, 0, 0, 0, 0, True);
+			if (moved)
+			{
+				char buf[64];
+				sprintf(buf, "Silent Move +%dp +%dp", x, y);
+
+				SendText(fd, buf, t->w);
+				XSync(dpy,0);
+			}
+			else
+			{
+				MoveResizePagerView(t, True);
+			}
+			SendText(fd, "Silent Raise", t->w);
+		}
+		if (do_switch_desk_later)
+		{
+			sprintf(command, "Silent MoveToDesk 0 %d", NewDesk +
+				desk1);
+			SendText(fd, command, t->w);
+			t->desk = NewDesk + desk1;
+		}
+		if (Scr.CurrentDesk == t->desk)
+		{
+			XSync(dpy,0);
+			usleep(5000);
+			XSync(dpy,0);
+
+			SendText(fd, "Silent FlipFocus NoWarp", t->w);
+		}
+	}
+	if (is_transient)
+	{
+		/* does not return */
+		ExitPager();
 	}
-	XSync(dpy,0);
-      }
-      else
-	MoveResizePagerView(t, True);
-      SendText(fd, "Silent Raise", t->w);
-    }
-    if (do_switch_desk_later)
-    {
-      sprintf(command,"Silent MoveToDesk 0 %d", NewDesk + desk1);
-      SendText(fd, command, t->w);
-      t->desk = NewDesk + desk1;
-    }
-    if(Scr.CurrentDesk == t->desk)
-    {
-      XSync(dpy,0);
-      usleep(5000);
-      XSync(dpy,0);
-
-      SendText(fd, "Silent FlipFocus NoWarp", t->w);
-    }
-  }
-  if (is_transient)
-  {
-    /* does not return */
-    ExitPager();
-  }
 }
 
 
@@ -2816,139 +2886,158 @@ void PictureIconWindow (PagerWindow *t)
   do_picture_window(t, t->IconView, t->icon_view_width, t->icon_view_height);
 }
 
-void IconMoveWindow(XEvent *Event,PagerWindow *t)
+void IconMoveWindow(XEvent *Event, PagerWindow *t)
 {
-  int x1,y1,finished = 0,wx,wy,n,x=0,y=0,xi=0,yi=0;
-  Window dumwin;
-  int m,n1,m1;
-  int moved = 0;
-  int KeepMoving = 0;
-  Window JunkRoot, JunkChild;
-  int JunkX, JunkY;
-  unsigned JunkMask;
+	int x1, y1, finished = 0, wx, wy, n, x = 0, y = 0, xi = 0, yi = 0;
+	Window dumwin;
+	int m, n1, m1;
+	int moved = 0;
+	int KeepMoving = 0;
+	Window JunkRoot, JunkChild;
+	int JunkX, JunkY;
+	unsigned JunkMask;
 
-  if(t==NULL)
-    return;
+	if (t == NULL || !t->allowed_actions.is_movable)
+	{
+		return;
+	}
 
-  n = Scr.VxMax / Scr.MyDisplayWidth;
-  m = Scr.VyMax / Scr.MyDisplayHeight;
-  n1 = (Scr.Vx + t->x) / Scr.MyDisplayWidth;
-  m1 = (Scr.Vy + t->y) / Scr.MyDisplayHeight;
-  wx = (Scr.Vx + t->x) * (icon_w - n) / Scr.VWidth + n1;
-  wy = (Scr.Vy + t->y) * (icon_h - m) / Scr.VHeight + m1;
+	n = Scr.VxMax / Scr.MyDisplayWidth;
+	m = Scr.VyMax / Scr.MyDisplayHeight;
+	n1 = (Scr.Vx + t->x) / Scr.MyDisplayWidth;
+	m1 = (Scr.Vy + t->y) / Scr.MyDisplayHeight;
+	wx = (Scr.Vx + t->x) * (icon_w - n) / Scr.VWidth + n1;
+	wy = (Scr.Vy + t->y) * (icon_h - m) / Scr.VHeight + m1;
 
-  XRaiseWindow(dpy,t->IconView);
+	XRaiseWindow(dpy, t->IconView);
 
-  XTranslateCoordinates(dpy, Event->xany.window, t->IconView,
-			Event->xbutton.x, Event->xbutton.y, &x1, &y1, &dumwin);
-  while(!finished)
-  {
-    FMaskEvent(dpy,ButtonReleaseMask | ButtonMotionMask|ExposureMask,Event);
+	XTranslateCoordinates(dpy, Event->xany.window, t->IconView,
+			      Event->xbutton.x, Event->xbutton.y, &x1, &y1,
+			      &dumwin);
+	while (!finished)
+	{
+		FMaskEvent(dpy, ButtonReleaseMask | ButtonMotionMask |
+			   ExposureMask, Event);
 
-    if(Event->type == MotionNotify)
-    {
-      x = Event->xbutton.x;
-      y = Event->xbutton.y;
-      if(moved == 0)
-      {
-	xi = x;
-	yi = y;
-	moved = 1;
-      }
+		if (Event->type == MotionNotify)
+		{
+			x = Event->xbutton.x;
+			y = Event->xbutton.y;
+			if (moved == 0)
+			{
+				xi = x;
+				yi = y;
+				moved = 1;
+			}
 
-      XMoveWindow(dpy,t->IconView, x - (x1),
-		  y - (y1));
-      if((x < -5)||(y < -5)||(x>icon_w+5)||(y>icon_h+5))
-      {
-	finished = 1;
-	KeepMoving = 1;
-      }
-    }
-    else if(Event->type == ButtonRelease)
-    {
-      x = Event->xbutton.x;
-      y = Event->xbutton.y;
-      XMoveWindow(dpy,t->PagerView, x - (x1),y - (y1));
-      finished = 1;
-    }
-    else if (Event->type == Expose)
-    {
-      HandleExpose(Event);
-    }
-  }
+			XMoveWindow(dpy, t->IconView, x - x1, y - y1);
+			if (x < -5 || y < -5 || x > icon_w + 5 ||
+			    y > icon_h + 5)
+			{
+				finished = 1;
+				KeepMoving = 1;
+			}
+		}
+		else if (Event->type == ButtonRelease)
+		{
+			x = Event->xbutton.x;
+			y = Event->xbutton.y;
+			XMoveWindow(dpy, t->PagerView, x - x1, y - y1);
+			finished = 1;
+		}
+		else if (Event->type == Expose)
+		{
+			HandleExpose(Event);
+		}
+	}
 
-  if(moved)
-  {
-    if((x - xi < MoveThreshold)&&(y - yi < MoveThreshold)&&
-       (x - xi > -MoveThreshold)&&(y -yi > -MoveThreshold))
-      moved = 0;
-  }
+	if (moved)
+	{
+		if (abs(x - xi) < MoveThreshold &&
+		    abs(y - yi) < MoveThreshold)
+		{
+			moved = 0;
+		}
+	}
 
-  if(KeepMoving)
-  {
-    char command[48];
-
-    if (FQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
-		      &x, &y, &JunkX, &JunkY, &JunkMask) == False)
-    {
-      /* pointer is on a different screen */
-      x = 0;
-      y = 0;
-    }
-    XUngrabPointer(dpy,CurrentTime);
-    XSync(dpy,0);
-    sprintf(command, "Silent Move %dp %dp", x, y);
-    SendText(fd, command, t->w);
-    SendText(fd, "Silent Raise", t->w);
-    SendText(fd, "Silent Move Pointer", t->w);
-  }
-  else
-  {
-    x = x - x1;
-    y = y - y1;
-    n1 = x * Scr.VWidth / (icon_w * Scr.MyDisplayWidth);
-    m1 = y * Scr.VHeight / (icon_h * Scr.MyDisplayHeight);
-    x = (x - n1) * Scr.VWidth / (icon_w - n) - Scr.Vx;
-    y = (y - m1) * Scr.VHeight / (icon_h - m) - Scr.Vy;
-    /* force onto desk */
-    if (x + t->icon_width + Scr.Vx < 0 )
-      x = - Scr.Vx - t->icon_width;
-    if (y + t->icon_height + Scr.Vy < 0)
-      y = - Scr.Vy - t->icon_height;
-    if (x + Scr.Vx >= Scr.VWidth)
-      x = Scr.VWidth - Scr.Vx - 1;
-    if (y + Scr.Vy >= Scr.VHeight)
-      y = Scr.VHeight - Scr.Vy - 1;
-    if((IS_ICONIFIED(t) && IS_ICON_STICKY_ACROSS_DESKS(t)) ||
-       IS_STICKY_ACROSS_DESKS(t))
-    {
-      if (x > Scr.MyDisplayWidth - 16)
-	x = Scr.MyDisplayWidth - 16;
-      if (y > Scr.MyDisplayHeight - 16)
-	y = Scr.MyDisplayHeight - 16;
-      if (x + t->width < 16)
-	x = 16 - t->width;
-      if (y + t->height < 16)
-	y = 16 - t->height;
-    }
-    if(moved)
-    {
-      if(IS_ICONIFIED(t))
-	XMoveWindow(dpy,t->icon_w,x,y);
-      else
-	XMoveWindow(dpy,t->w,x,y);
-      XSync(dpy,0);
-    }
-    else
-      MoveResizePagerView(t, True);
-    SendText(fd, "Silent Raise", t->w);
-    SendText(fd, "Silent FlipFocus NoWarp", t->w);
-  }
-  if (is_transient)
-  {
-    /* does not return */
-    ExitPager();
-  }
+	if (KeepMoving)
+	{
+		if (FQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
+				  &x, &y, &JunkX, &JunkY, &JunkMask) == False)
+		{
+			/* pointer is on a different screen */
+			x = 0;
+			y = 0;
+		}
+		XUngrabPointer(dpy, CurrentTime);
+		XSync(dpy, 0);
+		SendText(fd, "Silent Raise", t->w);
+		SendText(fd, "Silent Move Pointer", t->w);
+	}
+	else
+	{
+		x = x - x1;
+		y = y - y1;
+		n1 = x * Scr.VWidth / (icon_w * Scr.MyDisplayWidth);
+		m1 = y * Scr.VHeight / (icon_h * Scr.MyDisplayHeight);
+		x = (x - n1) * Scr.VWidth / (icon_w - n) - Scr.Vx;
+		y = (y - m1) * Scr.VHeight / (icon_h - m) - Scr.Vy;
+		/* force onto desk */
+		if (x + t->icon_width + Scr.Vx < 0 )
+		{
+			x = - Scr.Vx - t->icon_width;
+		}
+		if (y + t->icon_height + Scr.Vy < 0)
+		{
+			y = - Scr.Vy - t->icon_height;
+		}
+		if (x + Scr.Vx >= Scr.VWidth)
+		{
+			x = Scr.VWidth - Scr.Vx - 1;
+		}
+		if (y + Scr.Vy >= Scr.VHeight)
+		{
+			y = Scr.VHeight - Scr.Vy - 1;
+		}
+		if ((IS_ICONIFIED(t) && IS_ICON_STICKY_ACROSS_DESKS(t)) ||
+		   IS_STICKY_ACROSS_DESKS(t))
+		{
+			if (x > Scr.MyDisplayWidth - 16)
+			{
+				x = Scr.MyDisplayWidth - 16;
+			}
+			if (y > Scr.MyDisplayHeight - 16)
+			{
+				y = Scr.MyDisplayHeight - 16;
+			}
+			if (x + t->width < 16)
+			{
+				x = 16 - t->width;
+			}
+			if (y + t->height < 16)
+			{
+				y = 16 - t->height;
+			}
+		}
+		if (moved)
+		{
+			char buf[64];
+			sprintf(buf, "Silent Move +%dp +%dp", x, y);
+			SendText(fd, buf, t->w);
+			XSync(dpy, 0);
+		}
+		else
+		{
+			MoveResizePagerView(t, True);
+		}
+		SendText(fd, "Silent Raise", t->w);
+		SendText(fd, "Silent FlipFocus NoWarp", t->w);
+	}
+	if (is_transient)
+	{
+		/* does not return */
+		ExitPager();
+	}
 }
 
 
