@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /* This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -13,12 +14,14 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "config.h"
+
 #include "Tools.h"
 
 
-/*******************************************************/
-/* Fonction pour MiniScroll / Functions for MiniScroll */
-/*******************************************************/
+/*
+ * Fonction pour MiniScroll / Functions for MiniScroll
+ */
 void InitMiniScroll(struct XObj *xobj)
 {
   unsigned long mask;
@@ -73,7 +76,7 @@ void DestroyMiniScroll(struct XObj *xobj)
   XDestroyWindow(dpy,xobj->win);
 }
 
-void DrawMiniScroll(struct XObj *xobj)
+void DrawMiniScroll(struct XObj *xobj, XEvent *evp)
 {
 
   DrawReliefRect(-1, -1, xobj->width+2, xobj->height+2, xobj, hili, shad);
