@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /* This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -15,10 +16,10 @@
 
 #include "config.h"
 
-#include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <dirent.h>
 #include <pwd.h>
 #include "libs/ftime.h"
 #include <sys/stat.h>
@@ -38,7 +39,6 @@
 #include "Goodies.h"
 #include "FvwmTaskBar.h"
 #include "Mallocs.h"
-#include "Colors.h"
 #include "minimail.xbm"
 
 #define MAILCHECK_DEFAULT 10
@@ -47,12 +47,11 @@ extern Display *dpy;
 extern Window Root, win;
 extern int Fvwm_fd[2];
 extern int screen;
-extern char *Module;
+extern ModuleArgs *module;
 extern int win_width, win_height, win_y, win_border, RowHeight, Midline;
 extern rectangle screen_g;
 extern Pixel back, fore;
 extern int colorset;
-extern int Clength;
 extern GC blackgc, hilite, shadow, checkered;
 extern FlocaleWinString *FwinString;
 
@@ -65,6 +64,7 @@ int goodies_fontheight, clock_width;
 char *mailpath = NULL;
 char *clockfmt = NULL;
 char *datefmt = NULL;
+Bool do_display_clock = True;
 int BellVolume = DEFAULT_BELL_VOLUME;
 Pixmap mailpix = None;
 Pixmap wmailpix = None;
@@ -81,15 +81,16 @@ int IgnoreOldMail = False;
 int ShowTips = False;
 char *statusfont_string = NULL;
 int last_date = -1;
+Bool using_MailDir = False;
 
-void cool_get_inboxstatus();
+void cool_get_inboxstatus(void);
 
 #define gray_width  8
 #define gray_height 8
 extern unsigned char gray_bits[];
 
-/*                x  y  w  h  tw th open type *text   win */
-TipStruct Tip = { 0, 0, 0, 0,  0, 0,   0,   0, NULL, None };
+/*                x  y  w  h  tw th open type px  py *text   win */
+TipStruct Tip = { 0, 0, 0, 0,  0, 0,   0,   0, 0, 0,  NULL, None };
 
 
 static void CreateOrUpdateGoodyGC(void)
@@ -137,17 +138,48 @@ static void CreateOrUpdateGoodyGC(void)
       PictureBlackPixel(), PictureWhitePixel(), Pdepth);
     goodies_width += minimail_width + 7;
   }
-  else
+  else if (do_display_clock)
+  {
     goodies_width += 3;
+  }
+  else
+  {
+	  goodies_width += 0;
+  }
+
 }
 
 Bool change_goody_colorset(int cset, Bool force)
 {
-  if (cset < 0 || (cset != tipscolorset && !force))
-    return False;
-  CreateOrUpdateGoodyGC();
+	if (cset < 0)
+	{
+		return False;
+	}
+	if (cset == tipscolorset && Tip.win != None)
+	{
+		char *s;
 
-  return True;
+		if (Tip.text != NULL)
+		{
+			s = safestrdup(Tip.text);
+		}
+		else
+		{
+			s = NULL;
+		}
+		PopupTipWindow(Tip.px, Tip.py, s);
+		if (s != NULL)
+		{
+			free(s);
+		}
+	}
+	if (!force && cset != colorset)
+	{
+		return False;
+	}
+	CreateOrUpdateGoodyGC();
+
+	return True;
 }
 
 static char *goodyopts[] =
@@ -164,6 +196,7 @@ static char *goodyopts[] =
   "IgnoreOldMail",
   "ShowTips",
   "DateFormat",
+  "MailDir",
   NULL
 };
 
@@ -174,7 +207,7 @@ Bool GoodiesParseConfig(char *tline)
   char *option;
   int i;
 
-  option = tline + Clength;
+  option = tline + module->namelen+1;
   i = GetTokenIndex(option, goodyopts, -1, &rest);
   while (*rest && *rest != '\n' && isspace(*rest))
     rest++;
@@ -207,6 +240,11 @@ Bool GoodiesParseConfig(char *tline)
     break;
   case 3: /* ClockFormat */
     UpdateString(&clockfmt, rest);
+    do_display_clock = True;
+    if (clockfmt && clockfmt[0] == '\0' && datefmt && datefmt[0] == '\0')
+    {
+	    do_display_clock = False;
+    }
     break;
   case 4: /* StatusFont */
     CopyStringWithQuotes(&statusfont_string, rest);
@@ -235,9 +273,17 @@ Bool GoodiesParseConfig(char *tline)
     break;
   case 11: /* DateFormat */
     UpdateString(&datefmt, rest);
+    do_display_clock = True;
+    if (clockfmt && clockfmt[0] == '\0' && datefmt && datefmt[0] == '\0')
+    {
+	    do_display_clock = False;
+    }
+    break;
+  case 12: /* MailDir */
+    using_MailDir = True;
     break;
   default:
-    /* unknow option */
+    /* unknown option */
     return False;
   } /* switch */
 
@@ -247,9 +293,10 @@ Bool GoodiesParseConfig(char *tline)
 void LoadGoodiesFont(void)
 {
 	if ((FStatusFont =
-	     FlocaleLoadFont(dpy, statusfont_string, Module)) == NULL)
+	     FlocaleLoadFont(dpy, statusfont_string, module->name)) == NULL)
 	{
-		fprintf(stderr, "%s: Couldn't load font. Exiting!\n",Module);
+		fprintf(stderr, "%s: Couldn't load font. Exiting!\n",
+                       module->name);
 		exit(1);
 	}
 	goodies_fontheight = FStatusFont->height;
@@ -257,31 +304,42 @@ void LoadGoodiesFont(void)
 
 void InitGoodies(void)
 {
-  struct passwd *pwent;
-  char tmp[1024];
+	struct passwd *pwent;
+	char tmp[1024];
 
-  if (mailpath == NULL) {
-    strcpy(tmp, DEFAULT_MAIL_PATH);
-    pwent = getpwuid(getuid());
-    strcat(tmp, (char *) (pwent->pw_name));
-    UpdateString(&mailpath, tmp);
-  }
+	if (mailpath == NULL)
+	{
+		strcpy(tmp, DEFAULT_MAIL_PATH);
+		pwent = getpwuid(getuid());
+		strcat(tmp, (char *) (pwent->pw_name));
+		UpdateString(&mailpath, tmp);
+	}
 
-  CreateOrUpdateGoodyGC();
-  if (clockfmt)
-  {
-    struct tm *tms;
-    static time_t timer;
-    static char str[24];
-    time(&timer);
-    tms = localtime(&timer);
-    strftime(str, 24, clockfmt, tms);
-    clock_width = FlocaleTextWidth(FStatusFont, str, strlen(str)) + 4;
-  }
-  else
-    clock_width = FlocaleTextWidth(FStatusFont, "XX:XX", 5) + 4;
-  goodies_width += clock_width;
-  stwin_width = goodies_width;
+	CreateOrUpdateGoodyGC();
+	if (clockfmt)
+	{
+		if (!do_display_clock)
+		{
+			clock_width = 0;
+		}
+		else
+		{
+			struct tm *tms;
+			static time_t timer;
+			static char str[24];
+			time(&timer);
+			tms = localtime(&timer);
+			strftime(str, 24, clockfmt, tms);
+			clock_width = FlocaleTextWidth(
+				FStatusFont, str, strlen(str)) + 4;
+		}
+	}
+	else
+	{
+		clock_width = FlocaleTextWidth(FStatusFont, "XX:XX", 5) + 4;
+	}
+	goodies_width += clock_width;
+	stwin_width = goodies_width;
 }
 
 void Draw3dBox(Window wn, int x, int y, int w, int h, XRectangle *bounding)
@@ -324,6 +382,10 @@ void DrawGoodies(XEvent *evp)
 	int h = RowHeight;
 	Region t_region;
 
+	if (goodies_width == 0)
+	{
+		return;
+	}
 	if (evp)
 	{
 		if (!frect_get_intersection(
@@ -475,13 +537,14 @@ void CreateMailTipWindow()
 
 void RedrawTipWindow(void)
 {
-  /* FIXME: I am sure that we redraw to often */
+  /* FIXME: I am sure that we redraw too often */
   if (Tip.text) {
     FwinString->win = Tip.win;
     FwinString->gc = tipsgc;
     FwinString->str = Tip.text;
     FwinString->x = 3;
     FwinString->y = 2 + FStatusFont->ascent;
+    FwinString->flags.has_clip_region = False;
     if (tipscolorset >= 0)
     {
        FwinString->colorset = &Colorset[tipscolorset];
@@ -493,7 +556,7 @@ void RedrawTipWindow(void)
     }
     if (FftSupport && FStatusFont->fftf.fftfont != NULL)
     {
-       XClearArea(dpy, Tip.win, 0, 0, Tip.w, Tip.h, False);
+	    XClearArea(dpy, Tip.win, 0, 0, Tip.w, Tip.h, False);
     }
     FlocaleDrawString(dpy, FStatusFont, FwinString, 0);
     XRaiseWindow(dpy, Tip.win);
@@ -505,6 +568,8 @@ void PopupTipWindow(int px, int py, const char *text)
   int newx, newy;
   Window child;
 
+  Tip.px = px;
+  Tip.py = py;
   if (!ShowTips)
     return;
   if (Tip.win != None)
@@ -544,6 +609,7 @@ void ShowTipWindow(int open)
     if (Tip.win != None)
     {
       XMapRaised(dpy, Tip.win);
+      RedrawTipWindow();
     }
   }
   else
@@ -615,27 +681,18 @@ void CreateTipWindow(int x, int y, int w, int h)
     pchk = XCreatePixmapFromBitmapData(dpy, Tip.win, (char *)gray_bits,
 				       gray_width, gray_height, 1, 0, 1);
   gcval.stipple = pchk;
-  if (cgc)
-    XChangeGC(dpy, cgc, gcmask, &gcval);
-  else
-    cgc = fvwmlib_XCreateGC(dpy, pmask, gcmask, &gcval);
+  cgc = fvwmlib_XCreateGC(dpy, pmask, gcmask, &gcval);
 
   gcmask = GCForeground | GCBackground | GCGraphicsExposures | GCFillStyle;
   gcval.graphics_exposures = False;
   gcval.fill_style = FillSolid;
   gcval.foreground = 0;
   gcval.background = 0;
-  if (gc0)
-    XChangeGC(dpy, gc0, gcmask, &gcval);
-  else
-    gc0 = fvwmlib_XCreateGC(dpy, pmask, gcmask, &gcval);
+  gc0 = fvwmlib_XCreateGC(dpy, pmask, gcmask, &gcval);
 
   gcval.foreground = 1;
   gcval.background = 1;
-  if (gc1)
-    XChangeGC(dpy, gc1, gcmask, &gcval);
-  else
-    gc1 = fvwmlib_XCreateGC(dpy, pmask, gcmask, &gcval);
+  gc1 = fvwmlib_XCreateGC(dpy, pmask, gcmask, &gcval);
 
   XFillRectangle(dpy, pmask, gc0, 0, 0, w+4, h+4);
   XFillRectangle(dpy, pmask, cgc, 3, 3, w+4, h+4);
@@ -676,7 +733,7 @@ void DestroyTipWindow()
 /* (based on the code of 'coolmail' By Byron C. Darrah */
 /*-----------------------------------------------------*/
 
-void cool_get_inboxstatus()
+void cool_get_inboxstatus_mbox(void)
 {
    static off_t oldsize = 0;
    off_t  newsize;
@@ -718,6 +775,115 @@ void cool_get_inboxstatus()
    oldsize = newsize;
 }
 
+static int f_scandir(const char *dir, struct dirent *** namelist)
+{
+	DIR * f_dir;
+	struct dirent * f_temp;
+	/* 10 is just an arbituary number */
+	int f_count = 0, namelist_size = 10;
+	void * r_ret;
+
+	f_dir = opendir(dir);
+	if (f_dir == NULL)
+		return -1;
+	namelist[0] =
+		(struct dirent **)safemalloc(sizeof(struct dirent *) * 10);
+	do {
+		f_temp = readdir(f_dir);
+		if (f_temp == NULL)
+			break;
+		else if (strcmp(f_temp->d_name, ".") != 0 &&
+			 strcmp(f_temp->d_name, "..") != 0)
+		{
+			if (f_count > namelist_size)
+			{
+				r_ret = saferealloc(
+					(void *)(namelist[0]),
+					sizeof(struct dirent) * (f_count +10));
+				namelist[0] = (struct dirent **)r_ret;
+				namelist_size = f_count + 10;
+			}
+			namelist[0][f_count] = (struct dirent *)safemalloc(
+				sizeof(struct dirent));
+			memcpy(
+				namelist[0][f_count], f_temp,
+				sizeof(struct dirent));
+			f_count++;
+		}
+	} while (1);
+
+	return f_count;
+}
+
+void cool_get_inboxstatus_maildir(void)
+{
+	static int oldsize = 0;
+	int  newsize;
+	struct dirent **newlist;
+	struct dirent **curlist;
+	char * alt_mailpath;
+	int curent, newent;
+
+	alt_mailpath = safemalloc(strlen(mailpath) + strlen("/new") + 1);
+	strcpy(alt_mailpath, mailpath);
+	strcat(alt_mailpath, "/new");
+	newent = f_scandir(alt_mailpath, &newlist);
+	strcpy(alt_mailpath, mailpath);
+	strcat(alt_mailpath, "/cur");
+	curent = f_scandir(alt_mailpath, &curlist);
+	free(alt_mailpath);
+
+	newsize = curent + newent;
+
+	if (newent > 0)
+	{
+		anymail = 1;
+		unreadmail = 1;
+	}
+	else if (curent > 0)
+	{
+		anymail = 1;
+		unreadmail = 0;
+	}
+	else
+	{
+		anymail = 0;
+		unreadmail = 0;
+	}
+	if (newsize > oldsize && unreadmail)
+	{
+		newmail = 1;
+		mailcleared = 0;
+	}
+	else
+	{
+		newmail = 0;
+	}
+
+	oldsize = newsize;
+
+	while(curent--)
+	{
+		free(curlist[curent]);
+	}
+	free(curlist);
+	while(newent--)
+	{
+		free(newlist[newent]);
+	}
+	free(newlist);
+}
+
+void cool_get_inboxstatus()
+{
+	if (using_MailDir)
+	{
+		cool_get_inboxstatus_maildir();
+	} else {
+		cool_get_inboxstatus_mbox();
+	}
+}
+
 /*---------------------------------------------------------------------------*/
 
 void HandleMailClick(XEvent event)
